; -------------------------------------------------------------
; AT90S8515 USB HOST PROGRAM by M.yanagisawa.(MoroboshiLUM)
; HUB 1 system support
; HID 4 Device support
; Leaf/AquaPlus PIECE device proxy implemented
; -------------------------------------------------------------

.include "8515def.inc"

; Generic Register defination

;; Z = rs232c used
;; Y = usb register access used

.def	zero	= r2		; ZERO register
.def	one	= r3		; ONE register
.def	tgr9	= r4
.def	tgr8	= r5
.def	tgr7	= r6
.def	tgr6	= r7
.def	tgr5	= r8
.def	tgr4	= r9
.def	tgr3	= r10
.def	gr9	= r11
.def	gr8	= r12
.def	gr7	= r13
.def	gr6	= r14
.def	stack	= r15		; 
.def	gr0	= r16		; generic register 0
.def	gr1	= r17		; generic register 1
.def	gr2	= r18		; generic register 1
.def	gr3	= r19
.def	gr4	= r20
.def	gr5	= r21
.def	tgr0	= r22
.def	tgr1	= r23
.def	tgr2	= r24
.def	chipadr = r25		;
.def	chipdata = r26		;XL

.equ	ADRREQ	= 0x80
.equ	DATAREQ	= 0xC0

.equ	_CR	= 0x0D
.equ	_LF	= 0x0A

.equ	ENDPOINTS   = 4		; 
.equ	DEVADDR_ST  = 1
.equ	MAX_HUBPORTS = 4	; MAX 4 ports.
.equ	MAX_PAYLOADSIZE = 64	;
.equ	DEF_PAYLOADSIZE = 8	;
.equ	MAX_RETRY = 15000       ; + 65534

.equ	TIMER0_INITCNT = (255-41)
.equ	INTMASK = 0xdf

; Device Table Structure Offset
.equ	DevNum	 = 0
.equ	DevType	 = 1
.equ	DevSubType = 2
.equ	IntrEP	 = 3
.equ	DevAddr  = 4
.equ	DevSpeed = 5
.equ	DHubPorts = 6
.equ	DevEP	 = 7
.equ	DAttr	 =  11	; 6+4
.equ	DInterval = 15	; 10+4
.equ	DPayload =  19	; 14+4
.equ	DToggle  =  23	; 18+4

.equ	TIMEOUT0_ERR = 1
.equ	TIMEOUT1_ERR = 2
.equ	RETRY_ERR = 3
.equ	DISCONNECT_ERR = 3

.equ	DEV_TABLE_SIZE = (DToggle+ENDPOINTS)
.equ	MAX_DEVICES = 5
.equ	DEV_TABLE_SIZE_ALL = (DEV_TABLE_SIZE * (MAX_DEVICES-1))
;
; USB Standard Requests
; 
.equ	GET_STATUS	= 0
.equ	CLEAR_FEATURE	= 1
.equ	SET_FEATURE	= 3
.equ	SET_ADDRESS	= 5
.equ	GET_DESCRIPTOR	= 6
.equ	SET_DESCRIPTOR	= 7
.equ	GET_CONFIGURATION = 8
.equ	SET_CONFIGURATION = 9
.equ	GET_INTERFACE	= 10
.equ	SET_INTERFACE	= 11
.equ	SYNC_FRAME	= 12

.equ	DEV_DEVICE	= 1
.equ	DEV_CONFIGURATION = 2
.equ	DEV_STRING	= 3
.equ	DEV_INTERFACE	= 4
.equ	DEV_ENDPOINT	= 5

.equ	TYPE_OF_STANDARD = 0x00
.equ	TYPE_OF_CLASS	= 0x20
.equ	TYPE_OF_VENDOR	= 0x40

.equ	ISO_BIT	= 0x10

.equ	HID_DEVICE = 3
.equ	HUB_DEVICE = 9

.equ	UNIQUEDEV_STARTINDEX = 0x80
.equ	PIECE_DEVICE = (UNIQUEDEV_STARTINDEX + 0)
.equ	USBWR_DEVICE = (UNIQUEDEV_STARTINDEX + 1)

;; Defined Device Counts
.equ	DEVTABLE_COUNT = 2

;; P/ECE Manufacture ID & Product ID
.equ	PIECE_MID_H = 0x0E
.equ	PIECE_MID_L = 0x19
.equ	PIECE_PID_H = 0x10
.equ	PIECE_PID_L = 0x00

.equ	INTERVAL_COUNT = 10	; send piece interval

;; USBWriter Manufacture ID & Product ID
.equ	USBWR_MID_H = 0x04
.equ	USBWR_MID_L = 0xD7
.equ	USBWR_PID_H = 0x20
.equ	USBWR_PID_L = 0x00


;; Standard Device Descripter
.equ	dd_bLength = 0
.equ	dd_bDescriptorType = 1
.equ	dd_bcdUSB = 2
.equ	dd_bDeviceClass = 4
.equ	dd_bDeviceSubClass = 5
.equ	dd_bDeviceProtocol = 6
.equ	dd_bMaxPacketSize0 = 7
.equ	dd_idVendor = 8
.equ	dd_idProduct = 10
.equ	dd_bcdDevice = 12
.equ	dd_iManufacturer = 14
.equ	dd_iProduct = 15
.equ	dd_iSerialNumber = 16
.equ	dd_bNumConfigurations = 17

;; Standard Configuration Descripter
.equ	cd_bLength = 0
.equ	cd_bType = 1
.equ	cd_wLength = 2
.equ	cd_bNumIntf = 4
.equ	cd_bCV = 5
.equ	cd_bIndex = 6
.equ	cd_bAttr = 7
.equ	cd_bMaxPower = 8

;; Standard Interface Descriptor
.equ	id_bLength = 0
.equ	id_bType = 1
.equ	id_iNum = 2
.equ	id_iAltString = 3
.equ	id_bEndPoints = 4
.equ	id_iClass = 5
.equ	id_iSub = 6
.equ	id_iProto = 7
.equ	id_iIndex = 8

;; Standard EndPoint Descriptor
.equ	ed_bLength = 0
.equ	ed_bType = 1
.equ	ed_bEPAdd = 2
.equ	ed_bAttr = 3
.equ	ed_wPayLoad = 4
.equ	ed_bInterval = 6

;; HID Standard Descriptor
.equ	hid_bLength = 0
.equ	hid_bDescriptorType = 1
.equ	hid_bcdHID = 2
.equ	hid_bCountryCode = 4
.equ	hid_bNumDescriptors = 5
.equ	hid_bDescriptorType2 = 6
.equ	hid_wItemLength = 7

;; Hub Standard Descriptor
.equ	hub_bDescLength	= 0
.equ	hub_bDescriptorType = 1
.equ	hub_bNbrPorts = 2
.equ	hub_wHubCharacteristics = 3
.equ	hub_bPwrOn2PwrGood = 5
.equ	hub_bHubContrCurrent = 6
.equ	hub_DeviceRemovable = 7
;;.equ	hub_PortPwrCtrlMask

; -------------------------------------------------------------------------
; Class Descriptor for HID (chap.7) : These values from bRequest
; -------------------------------------------------------------------------
; HID Class-Specific Request
.equ	GET_REPORT = 0x01
.equ	GET_IDLE   = 0x02
.equ	GET_PROTOCOL = 0x03

.equ	SET_REPORT = 0x09
.equ	SET_IDLE   = 0x0A
.equ	SET_PROTOCOL = 0x0B

.equ	HID_DEV	   = 0x21
.equ	HID_REPORT = 0x22
.equ	HID_PHYSICAL = 0x23

; Report Type
.equ	INPUT_REPORT	= 0x01
.equ	OUTPUT_REPORT   = 0x02
.equ	FEATURE_REPORT = 0x03

; Protocol Type
.equ	BOOT_PROTOCOL = 0x00
.equ	REPORT_PROTOCOL = 0x01

; Protocol Codes in the bInterfaceProtocol
.equ PROTOCOL_NONE  = 0
.equ PROTOCOL_KEYBOARD =1
.equ PROTOCOL_MOUSE = 2
.equ PROTOCOL_OTHER = 3

; SubClass Codes in the bInterfaceSubClass
.equ sCLASS = 1
.equ sOTHER = 2

; ---------------------------------------------------------
; Hub Descriptor (USB 1.1 chap.11)
; ---------------------------------------------------------
.equ	CLEAR_HUB_FEATURE = 0x20
.equ	CLEAR_PORT_FEATURE = 0x23
.equ	GET_BUS_STATE = 0xA3
.equ	GET_HUB_DESCRIPTOR = 0xA0
.equ	GET_HUB_STATUS = 0xA0
.equ	GET_PORT_STATUS = 0xA3
.equ	SET_HUB_DESCRIPTOR = 0x20
.equ	SET_HUB_FEATURE = 0x20
.equ	SET_PORT_FEATURE = 0x23

.equ	C_HUB_OVER_CURRENT = 1
.equ	PORT_CONNECTION = 0
.equ	PORT_ENABLE = 1
.equ	PORT_SUSPEND = 2
.equ	PORT_OVER_CURRENT = 3
.equ	PORT_RESET = 4
.equ	PORT_POWER = 8
.equ	PORT_LOW_SPEED = 9
.equ	C_PORT_CONNECTION = 16
.equ	C_PORT_ENABLE = 17
.equ	C_PORT_SUSPEND = 18
.equ	C_PORT_OVER_CURRENT = 19
.equ	C_PORT_RESET = 20

; ---------------------------------------
; SL811HS definations
; from Application notes
; ---------------------------------------
; Control registers

.equ	EP0Control = 0x00
.equ	EP0Address = 0x01
.equ	EP0XferLen = 0x02
.equ	EP0Status = 0x03
.equ	EP0Counter = 0x04

.equ	EP1Control = 0x08
.equ	EP1Address = 0x09
.equ	EP1XferLen = 0x0a
.equ	EP1Status = 0x0b
.equ	EP1Counter = 0x0c

.equ	CtrlReg	= 0x05
.equ	IntEna	= 0x06
.equ	IntStatus = 0x0d
.equ	DATASet = 0x0e
.equ	CSOFcnt = 0x0f

; interrupt status
.equ	USBA_done = 0x01
.equ	USBB_done = 0x02
.equ	BabbeDetect = 0x04
.equ	SOFtimer = 0x10
.equ	InsertRemove = 0x20
.equ	USBResetResume = 0x40
.equ	DplusVal = 0x80

.equ	DATA0_WR = 0x07		; Arm+Enable+transmit to Host+DATA0
.equ	DATA1_WR = 0x05		; Arm+Enable+transmit to Host+DATA1
.equ	ZDATA0_WR = 0x47	; Arm+Transaction Ignored+transmit to Host+DATA0
.equ	ZDATA1_WR = 0x45	; Arm+Transaction Ignored+transmit to Host+DATA1
.equ	DATA0_RD = 0x03		; Arm+Enable+receive from Host+DATA0
.equ	DATA1_RD = 0x43		; Arm+Enable+receive from Host+DATA1
.equ	PREAMBLE = 0x80		; Preamble bit
.equ	PID_SOF = 0xa5		; SOF Token
.equ	PID_SETUP = 0x2d	; SETUP Token
.equ	PID_IN = 0x69		; IN Token
.equ	PID_OUT = 0xe1		; OUT Token
.equ	PID_PRE = 0x3c		; PRE Token
.equ	PID_NAK	= 0x5a		; NAK Token
.equ	PID_STALL = 0x1e	; STALL Token
.equ	PID_DATA0 = 0xc3	; DATA0 Token
.equ	PID_DATA1 = 0x4b	; DATA1 Token

.equ	cMemStart = 0x10	; Host Memory Start
.equ	ubufA = 0x80		; Buffer A address for DATA0
.equ	ubufB = 0xc0		; Buffer B address for DATA1
.equ	uxferLen = 0x40		; Xfer Length
.equ	sMemSize = 0xc0		; Total SL811HS memory size
.equ	cMemEnd = 256		; Upper limit of memory

.equ	EP0Buf	= 0x40
.equ	EP0Len	= 0x40

.equ	SL811RES = 4		; PORTD bit 4
.equ	USBPOWER = 5		; PORTD bit 5

	.CSEG
	.ORG	0

; reset vector

	rjmp	RESET		; Reset handle
	rjmp	USBINT		; Ext int0
	rjmp	RESET		; Ext int1
	rjmp	RESET		; Timer1 capture handler
	rjmp	RESET		; Timer1 compA handler
	rjmp	TIMERINTB	; Timer1 compB handler
	rjmp	RESET		; Timer1 ov1 handler
	rjmp	TIMERINT0	; Timer0 ov1 handler
	rjmp	RESET		; spi handler
	rjmp	RXINT		; uart rx handler
	rjmp	RESET		; uart dre handler
	rjmp	RESET		; uart txc handler
	rjmp	RESET		; anacomp handler

; ----------------------------------------
; Serial Receviced interrupt
; ----------------------------------------
RXINT:
	in	stack,SREG	; SREG backup
	push	gr0
	push	XH
	push	XL

	in	gr0,USR
	andi	gr0,0x18
	brne	rxerr
	in	gr0,UDR		; Receive data
	lds	XL,rxwptr	; read write pointer
	st	X+,gr0		; store buffer
	andi	XL,0x6f		; pointer mask
	lds	gr0,rxrptr	; load read pointer
	cp	gr0,XL		; buffer overflow?
	breq	rxerr		;
	sts	rxwptr,XL	; refresh pointer
rxerr:
	pop	XL
	pop	XH
	pop	gr0

	out	SREG,stack	; SREG restore

	reti			;

; ----------------------------------------------
; Timer 0 Overflow Interrupt
; 1msec
; ----------------------------------------------
TIMERINT0:
	in	stack,SREG
	push	XL		; 
	push	XH

	in	XL,PORTB
	andi	XL,0x01
	breq	tim_x
	cbi	PORTB,0
	rjmp	tim_xx
tim_x:
	sbi	PORTB,0
tim_xx:
	ldi	XL,TIMER0_INITCNT
	out	TCNT0,XL	; Counter Reset
	ldi	XL,low(IntrCounters)
	ldi	XH,high(IntrCounters)
;; counter 0
	ld	tgr2,X
	sub	tgr2,one
	brcs	PC+2
	st	X,tgr2
;; counter 1
	adiw	XL,1
	ld	tgr2,X
	sub	tgr2,one
	brcs	PC+2
	st	X,tgr2
;; counter 2
	adiw	XL,1
	ld	tgr2,X
	sub	tgr2,one
	brcs	PC+2
	st	X,tgr2
;; counter 3
	adiw	XL,1
	ld	tgr2,X
	sub	tgr2,one
	brcs	PC+2
	st	X,tgr2
;; counter 4
	adiw	XL,1
	ld	tgr2,X
	sub	tgr2,one
	brcs	PC+2
	st	X,tgr2
	pop	XH
	pop	XL		;
	out	SREG,stack
	reti

; ----------------------------------------------
; Timer Interrupt
; 0.1sec
; ----------------------------------------------
TIMERINTB:
	in	stack,SREG
	push	XL		; 
	out	TCNT1H,zero	; MUST setting High Register at FIRST
	out	TCNT1L,zero
	lds	XL,timercnt
	inc	XL		; +1 (0-255-0)
	sts	timercnt,XL
	pop	XL		;
	out	SREG,stack
	reti

; ----------------------------------------------
; USB Interrupt
; ----------------------------------------------
USBINT:
	in	stack,SREG
;;; Device Insert/Remove checking only.
	out	SREG,stack
	reti

; =======================================================================

RESET:
;; initializing ports
	cli
	
	ldi	gr0,high(RAMEND)
	out	SPH,gr0
	ldi	gr0,low(RAMEND)
	out	SPL,gr0

	clr	zero		;; init zero register
	clr	one		;; 
	inc	one		;; one = 1

	;; interrupt
	ldi	gr0,0		;; all disable
	out	GIMSK,gr0	
	ldi	gr0,(1<<OCIE1B)	| (1<<TOIE0) ;; timer enable
	out	TIMSK,gr0

	; clock duty = CK/256
	ldi	gr0,(1<<CS02)
	out	TCCR0,gr0
	
	; clock duty = CK/1024
	ldi	gr0,(1<<CS12) | (1<<CS10)
	out	TCCR1B,gr0

	; compare = 500 count  0.1sec
	ldi	gr0,high(500)
	out	OCR1BH,gr0
	ldi	gr0,low(500)
	out	OCR1BL,gr0

	; Counter Clear
	ldi	gr0,TIMER0_INITCNT	; 
	out	TCNT0,gr0	; 

	; Timer 1 counter clear
	out	TCNT1H,zero	; 
	out	TCNT1L,zero

	sts	timercnt,zero	;; clear count

	ldi	gr0,(1 << SRE)  ;; SRAM I/F enable
	out	MCUCR,gr0

	ldi	gr0,(1 << ACD)  ;; Analog Comp disable
	out	ACSR,gr0

	;; rx interrupt enable, 
	ldi	gr0,(1 << RXCIE) | (1 << RXEN) | (1 << TXEN)
	out	UCR,gr0
	
	ldi	gr0,7		;; 38400bps@4.912MHz
	out	UBRR,gr0

	ldi	gr0,low(rxbuf)
	sts	rxwptr,gr0	;; 
	sts	rxrptr,gr0	;; X = 0x60

	; output port a,b,c
	ldi	gr0,0xFF
	out	DDRA,gr0
	out	DDRB,gr0
	out	DDRC,gr0
	out	DDRD,gr0

	; disable output
	ldi	gr0,0xFF
	out	PORTA,gr0
	out	PORTB,gr0
	out	PORTC,gr0
	ldi	gr0,0xef	;; VBUS OFF/RESET
	out	PORTD,gr0

	; clear counters
	ldi	XL,low(IntrCounters)
	ldi	XH,high(IntrCounters)
	ldi	gr0,MAX_DEVICES
intcnt_loop:
	st	X+,zero
	dec	gr0
	brne	intcnt_loop

	rcall	wait50us_
	sbi	PORTD,SL811RES	;; RESET OFF 
	rcall	wait50us_

; --------------------------------------------------
main:
	rcall	InitValiables	; initializing tables 
	rcall	initmodule
	brcc	PC+2
	rjmp	connection_wait_nomsg

	rcall	FindUSBDevices
	rcall	InitHubDevice
	rcall	InitKeyDevices
	rcall	InitMouseDevices
;;	rcall	DispKeyboard
	
main_loop:
	clr	gr0		; Device number
main_devloop:
;; check reset counter
	lds	gr1,disconnected ; Status Check
	or	gr1,zero
	breq	PC+2
	rjmp	connection_wait
	ldi	chipadr,IntStatus
	rcall	USBRead_
	andi	chipdata,0x20	; Removed ?
	breq	PC+2
	rjmp	connection_wait
	ldi	XL,low(RstCounters)
	ldi	XH,high(RstCounters)
	add	XL,gr0
	adc	XH,zero
	ld	gr1,X
	or	gr1,zero	; no device ?
	breq	main_nodevice
;; check interrupt counter
	ldi	XL,low(IntrCounters)
	ldi	XH,high(IntrCounters)
	add	XL,gr0
	adc	XH,zero
	ld	gr1,X
	cp	gr1,zero
	breq	main_executedevice_intr_read
main_nodevice:
	inc	gr0
	cpi	gr0,MAX_DEVICES
	brne	main_devloop
	rjmp	main_loop

	; counter overflow
	; Interrupt Transfer Execution
main_executedevice_intr_read:
	push	gr0
	mov	tgr1,gr0	; backup device
	rcall	GetBaseTblAdr	; Y = Table Address
	ldd	gr0,Y+DevType
	cpi	gr0,HUB_DEVICE
	brne	main_nothubreq
;; check hub status
	rcall	ReadHubData	; return gr0 : port change bit
	or	gr0,zero
	breq	main_devloop_0
;; check device status.
;	push	gr0
;	mov	gr1,gr0
;	ldi	gr0,'R'
;	rcall	senddata_
;	rcall	dispresult
;	pop	gr0
	rcall	RefreshHubStatus
	rjmp	main_devloop_0

main_nothubreq:
	cpi	gr0,HID_DEVICE
	brne	main_devloop_0
;; Key or Mouse Device
	ldd	gr0,Y+DevSubType
	cpi	gr0,(PROTOCOL_KEYBOARD << 4) | 1
	brne	main_nokeydevice
	rcall	ExecuteKeyRead
	cpi	gr0,0xff
	breq	main_devloop_0
	rcall	senddata_
;	cpi	gr0,'0'
;	brne	main_devloop_0
;	rcall	SetRTC_PIECE
	mov	gr1,gr0
	rcall	WriteKey_PIECE
	rjmp	main_devloop_0
main_nokeydevice:
	cpi	gr0,(PROTOCOL_MOUSE << 4) | 1
	brne	main_devloop_0
	rcall	ExecuteMouseRead
	brcs	main_devloop_0
	rcall	DispMouseStatus
	rcall	WriteKey_PIECE_Mouse
main_devloop_0:
	pop	gr0
	ldi	XL,low(RstCounters)
	ldi	XH,high(RstCounters)
	add	XL,gr0
	adc	XH,zero
	ld	gr1,X
;; reset counter
	ldi	XL,low(IntrCounters)
	ldi	XH,high(IntrCounters)
	add	XL,gr0
	adc	XH,zero
	st	X,gr1

	rjmp	main_nodevice

; -------------------------------------------
; Initialize KEYBOARD Device
; gr0 : Device Number
; -------------------------------------------
InitKeyboard:
	rcall	GetBaseTblAdr	; Y = Table Address
	ldi	XL,low(RstCounters)
	ldi	XH,high(RstCounters)
	ld	gr0,Y		; load device num
	mov	gr1,gr0
	add	XL,gr0
	adc	XH,zero		;
	ldd	gr0,Y+IntrEP
	push	YL
	push	YH
	add	YL,gr0
	adc	YH,zero
	ldd	gr0,Y+DInterval
	pop	YH
	pop	YL
	lsr	gr0
	inc	gr0		; minimum.1
	st	X,gr0		; set Interval Counter
;;
	ldi	XL,low(PrevKey)
	ldi	XH,high(PrevKey)
	add	XL,gr1
	adc	XH,zero		;
	st	X,zero		; Clear Previous Key
;;
	ldi	gr0,BOOT_PROTOCOL
	clr	gr1
	rcall	SetProtocol	; set boot protocol
	clr	gr0
	mov	gr1,one
	rcall	SetIdle		; idle minimum

	ret

; -------------------------------------------
; Initialize HUB Device
; -------------------------------------------
InitHUBDevice:
	lds	gr0,HubDevNum
	cpi	gr0,0xff
	brne	PC+2
	ret
	rcall	GetBaseTblAdr	; Y = Table Address
	ldi	XL,low(RstCounters)
	ldi	XH,high(RstCounters)
	ld	gr0,Y		; load device num
	mov	gr1,gr0
	add	XL,gr0
	adc	XH,zero		;
	ldd	gr0,Y+IntrEP
	push	YL
	push	YH
	add	YL,gr0
	adc	YH,zero
	ldd	gr0,Y+DInterval
	pop	YH
	pop	YL
	lsr	gr0
	inc	gr0		; minimum.1
	st	X,gr0		; set Interval Counter

	ret


; -------------------------------------------
; Initialize Mouse Device
; gr0 : Device Number
; -------------------------------------------
InitMouse:
	rcall	GetBaseTblAdr	; Y = Table Address
	ldi	XL,low(RstCounters)
	ldi	XH,high(RstCounters)
	ld	gr0,Y		; load device num
	mov	gr1,gr0		; backup
	add	XL,gr0
	adc	XH,zero		;
	ldd	gr0,Y+IntrEP
	push	YL
	push	YH
	add	YL,gr0
	adc	YH,zero
	ldd	gr0,Y+DInterval
	pop	YH
	pop	YL
	lsr	gr0
	inc	gr0		;
	st	X,gr0		; set Interval Counter

	ldi	XL,low(PrevButton)
	ldi	XH,high(PrevButton)
	mov	gr0,gr1
	lsl	gr1
	add	gr1,gr0
	add	XL,gr1
	adc	XH,zero		;
	st	X+,zero		; Clear Previous Button
	st	X+,zero		; Clear Previous Button
	st	X+,zero		; Clear Previous Button
	sts	STATUSCNT,zero	; Clear Count

	ldi	gr0,BOOT_PROTOCOL
	clr	gr1
	rcall	SetProtocol	; set boot protocol
	clr	gr0
	ldi	gr1,10		; one
	rcall	SetIdle		; idle minimum

	ret


; ---------------------------------------------------
; Binary to Decimal Ascii (signed)
; gr0 : data
; X   : store buffer
; ---------------------------------------------------
BinToDec:
	push	gr1
	push	gr2
	
	ldi	gr1,'+'
	cpi	gr0,0x80	;; plus or minus
	brcs	BinToDec_Plus
;; minus
	neg	gr0
	ldi	gr1,'-'
BinToDec_Plus:
	st	X+,gr1
	ldi	gr1,0x20	;; space
	st	X+,gr1
	st	X+,gr1
	st	X+,gr1
	st	X,zero
;; sign___NULL pattern,
;;
BinToDec_loop0:
	clr	gr2		;; count
BinToDec_loop:
	mov	gr1,gr0
	subi	gr1,10		;;
	brcs	BinToDec_SetChr
	inc	gr2		
	mov	gr0,gr1
	rjmp	BinToDec_loop
BinToDec_SetChr:
	ori	gr0,0x30	;; ASCII
	st	-X,gr0
	mov	gr0,gr2
	cp	gr0,zero
	brne	BinToDec_loop0
BinToDec_exit:

	pop	gr2
	pop	gr1

	ret

; ---------------------------------------------------
; disp mouse status
; gr0 : button info
; gr1 : x axis
; gr2 : y axis
; ---------------------------------------------------
DispMouseStatus:
	push	gr0
	push	gr1
	push	gr2
	mov	gr3,gr0
	ldi	gr0,'L'
	sbrs	gr3,0
	ldi	gr0,'_'
	rcall	senddata_
	ldi	gr0,'R'
	sbrs	gr3,1
	ldi	gr0,'_'
	rcall	senddata_
	ldi	gr0,' '
	rcall	senddata_
	ldi	gr0,'X'
	rcall	senddata_
	ldi	gr0,':'
	rcall	senddata_
	ldi	XH,high(DescriptorBuf)
	ldi	XL,low(DescriptorBuf)
	mov	gr0,gr1
	rcall	BinToDec
	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	rcall	sendline_ram
	ldi	gr0,' '
	rcall	senddata_
	ldi	gr0,'Y'
	rcall	senddata_
	ldi	gr0,':'
	rcall	senddata_
	ldi	XH,high(DescriptorBuf)
	ldi	XL,low(DescriptorBuf)
	mov	gr0,gr2
	rcall	BinToDec
	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	rcall	sendline_ram
	rcall	sendcrlf_
	pop	gr2
	pop	gr1
	pop	gr0

	ret

; ---------------------------------------------------
; Connection Check
; ---------------------------------------------------
connection_wait:
	ldi	ZH,high(disconnect_msg*2)
	ldi	ZL,low(disconnect_msg*2)
	rcall	sendline_		; nodevice
connection_wait_nomsg:
	ldi	chipadr,IntStatus
	ldi	chipdata,0xff
	rcall	USBWrite_
connection_waitloop:
	rcall	USBRead_
	andi	chipdata,0x20
	breq	connection_waitloop
;; changed
	rjmp	main


; --------------------------------------------------
; Initialize all keyboard device
; --------------------------------------------------
InitKeyDevices:
	clr	gr0
InitKeyDevices_loop:
	ldi	gr1,PROTOCOL_KEYBOARD
	rcall	GetHIDDeviceIndex
	cpi	gr0,0xff
	breq	InitKeyDevices_fin
	push	gr0
	rcall	InitKeyboard
	pop	gr0
	inc	gr0
	cpi	gr0,MAX_DEVICES
	brne	InitKeyDevices_loop
InitKeyDevices_fin:

	ret

; --------------------------------------------------
; Initialize all mouse device
; --------------------------------------------------
InitMouseDevices:
	clr	gr0
InitMouseDevices_loop:
	ldi	gr1,PROTOCOL_MOUSE
	rcall	GetHIDDeviceIndex
	cpi	gr0,0xff
	breq	InitMouseDevices_fin
	push	gr0
	rcall	InitMouse
	pop	gr0
	inc	gr0
	cpi	gr0,MAX_DEVICES
	brne	InitMouseDevices_loop
InitMouseDevices_fin:

	ret

; ---------------------------------
; Find Selected Hub Port Device
; gr0 : Port num (in) 1 base
;
; gr0 : Device Number(out)
; Y : Dev Table address
; ---------------------------------
FindSelectedHubPortDev:
	mov	gr1,gr0		; port num
	ldi	gr2,0
FindSelHubPortDev_loop:
	mov	gr0,gr2
	rcall	GetBaseTblAdr	; Y = Table Address
	ldd	gr3,Y+DevType
	or	gr3,zero	; type zero is illegal
	breq	FindSelHubPortDev_skip
	ldd	gr3,Y+DHubPorts	; Load Port number
	cp	gr1,gr3
	brne	PC+3		; 
	ld	gr0,Y		; dev numver
	ret			; find
FindSelHubPortDev_skip:
	inc	gr2
	cpi	gr2,MAX_DEVICES
	brne	FindSelHubPortDev_loop
	ser	gr0
	ret

; ------------------------------------
; Set Hub Port Stat
; gr0 : port
; ------------------------------------
SetHubPortStat:
	push	gr1
	ldi	gr1,1
	cp	gr0,zero
	breq	SetHubPortStat_exit
SetHubPortStat_loop:
	lsl	gr1
	dec	gr0
	cpi	gr0,1
	brne	SetHubPortStat_loop
SetHubPortStat_exit:
	lds	gr0,HubPorts
	or	gr0,gr1
	sts	HubPorts,gr0
	pop	gr1
	
	ret

; ------------------------------------
; Clear Hub Port Stat
; gr0 : port
; ------------------------------------
ClearHubPortStat:
	push	gr1
	ldi	gr1,1
	cp	gr0,zero
	breq	ClearHubPortStat_exit
ClearHubPortStat_loop:
	lsl	gr1
	dec	gr0
	cpi	gr0,1
	brne	ClearHubPortStat_loop
ClearHubPortStat_exit:
	lds	gr0,HubPorts
	com	gr1
	and	gr0,gr1
	sts	HubPorts,gr0
	pop	gr1
	
	ret

; -------------------------------------------
; Set Reset Counter
; -------------------------------------------
SetResetCounter:
	ldi	XL,low(RstCounters)
	ldi	XH,high(RstCounters)
	push	gr0
	ld	gr0,Y			;; Device Number
	add	XL,gr0
	adc	XH,zero
	pop	gr0
	st	X,gr0			;; Clear Reset Counter

	ret

; ---------------------------------
; Refresh Hub Status
; gr0 : now changed bit
; ---------------------------------
RefreshHubStatus:
	mov	tgr4,gr0
	mov	tgr3,one
	lsr	tgr4	;;
RefreshHubStat_loop:
	lsr	tgr4
	brcs	PC+2
	rjmp	RefreshHubStat_nochange
	mov	gr0,tgr3
	rcall	FindSelectedHubPortDev
	;; Y = Device Table
	cpi	gr0,0xFF
	breq	RefreshHubStat_Insert
;; Remove ? 
	mov	gr1,tgr3
	rcall	GetHubPortStatus
	ldd	gr0,Z+2
	push	ZH
	push	ZL
	rcall	ClearAllChangeBit
	pop	ZL
	pop	ZH
	ld	gr1,Z
	andi	gr1,0x02		;; Device Enable?
	brne	RefreshHubStat_nochange ;; If Enabled then nochange
;;
	ser	gr1
	ldd	gr0,Y+DevType
	cpi	gr0,PIECE_DEVICE
	brne	RefreshHubStat_notpiece
	sts	PIECEDEVNUM,gr1		;; Clear!
	rjmp	RefreshHubStat_clearinfo
RefreshHubStat_notpiece:
	cpi	gr0,USBWR_DEVICE
	brne	RefreshHubStat_clearinfo
	sts	USBWRDEVNUM,gr1		;; Clear!
RefreshHubStat_clearinfo:
	mov	gr0,tgr3
	rcall	ClearHubPortStat	;; HubPorts clear
	mov	gr1,tgr3
	rcall	DisableHubPort		;; Port Disable
	;; Clear Interrupt Transfer Clear
	clr	gr0			;; 
	rcall	SetResetCounter
	mov	tgr1,gr0
	rcall	ClearDeviceInfo		;; clear Device info
	push	gr0
	push	ZL
	push	ZH
	ldi	ZH,high(disconnect_msg*2)
	ldi	ZL,low(disconnect_msg*2)
	rcall	sendline_		; nodevice
	pop	ZH
	pop	ZL
	pop	gr0
	rjmp	RefreshHubStat_nochange
RefreshHubStat_Insert:
	;; insert ?
	mov	gr1,tgr3
	rcall	QueryUSBDevice  ;; Get USB Device
;; initialize device
	ldd	gr0,Y+DevType
	cpi	gr0,HID_DEVICE	; Check HID
	brne	RefreshHubStat_noinit
	ldd	gr0,Y+IntrEP			; Intr Endpoint exist?
	or	gr0,zero
	breq	RefreshHubStat_noinit
	ldd	gr0,Y+DevSubType
	cpi	gr0,(PROTOCOL_KEYBOARD << 4) | 1
	breq	RefreshHubStat_initkey
	cpi	gr0,(PROTOCOL_MOUSE << 4) | 1
	brne	RefreshHubStat_noinit
;; mouse
	ldd	gr0,Y+DevNum
	rcall	InitMouse
	rjmp	RefreshHubStat_noinit
RefreshHubStat_initkey:
	ldd	gr0,Y+DevNum
	rcall	InitKeyboard
RefreshHubStat_noinit:
	lds	gr0,HubDevNum
	rcall	GetBaseTblAdr
RefreshHubStat_nochange:
	inc	tgr3
	ldi	gr0,HubPortNum
	inc	gr0
	cp	tgr3,gr0
	brcc	PC+2
	rjmp	RefreshHubStat_loop
;; fin

	ret

; --------------------------------------------------
; --------------------------------------------------
USBRead_:
	rjmp	USBRead
USBWrite_:
	rjmp	USBWrite
wait50us_:
	rjmp	wait50us
senddata_:
	rjmp	senddata
sendline_:
	rjmp	sendline
sendcrlf_:
	rjmp	sendcrlf

; --------------------------------------------------
; --------------------------------------------------


; ---------------------------------
; AVR Memory Initialize
; ---------------------------------
InitValiables:
	cli
	
	;; device table all clear!
	ldi	ZL,low(DEVICE_TABLE)
	ldi	ZH,high(DEVICE_TABLE)
	ldi	gr0,DEV_TABLE_SIZE*5
initval_loop:
	st	Z+,zero
	dec	gr0
	brne	initval_loop

	sts	HubPortNum,zero
	sts	HubPorts,zero
	sts	KEYSTATUS,zero

	ser	gr0
	sts	HubDevNum,gr0	;; 0xff
	sts	PIECEDEVNUM,gr0
	sts	USBWRDEVNUM,gr0

	ldi	gr0,DEVADDR_ST
	sts	DEVADDR_CNT,gr0

	sts	enablenak,zero
	sts	disconnected,zero

	; clear reset counters
	ldi	XL,low(RstCounters)
	ldi	XH,high(RstCounters)
	ldi	gr0,MAX_DEVICES
initval_loop1:
	st	X+,zero
	dec	gr0
	brne	initval_loop1

	sei

	ret

; ---------------------------------
; Initializing Host Mode
; ---------------------------------
Initmodule:
	ldi	chipdata,0xAE
	ldi	chipadr,CSOFcnt		; enter host mode
	rcall	USBWrite
	in	gr0,PORTD
	andi	gr0,(1 << USBPower)
	breq	initmodule_alreadypoweron
	cbi	PORTD,USBPower		; VBUS Power up
;	rcall	wait10ms
;	rcall	wait10ms
;	rcall	wait10ms		; wait 30ms
	ldi	gr2,200
	rcall	wait10xms
initmodule_alreadypoweron:
	rcall	MemoryTest
	or	gr1,gr2
	breq	Init_ok			; if error, hangup
	sec
	ret				; error...

init_ok:
	ldi	chipadr,IntEna		; Interrupt register set.
	ldi	chipdata,0x63		; USB-A/B,Insert/Remove,USB Reset
	rcall	USBWrite		;
	ldi	chipdata,0xAE
	ldi	chipadr,CSOFcnt		; enter host mode
	rcall	USBWrite
	ldi	chipadr,CtrlReg
	ldi	chipdata,0x48		; USB engine reset
	rcall	USBWrite
	clr	chipdata
	rcall	USBWrite		; reset clear
;;;;
	ldi	chipdata,0xAE
	ldi	chipadr,CSOFcnt		; enter host mode
	rcall	USBWrite
	ldi	chipadr,CtrlReg
	ldi	chipdata,0x48		; USB engine reset
	rcall	USBWrite
	rcall	wait10ms
	clr	chipdata
	rcall	USBWrite		; reset clear
	rcall	wait1ms
	ldi	chipadr,IntStatus
	ldi	chipdata,0xff		; all clear
	rcall	USBWrite
	ldi	chipadr,IntStatus
	rcall	USBRead			; Low/High speed device detect
	sbrs	chipdata,6
	rjmp	detect_Device
	sts	disconnected,one
	ldi	ZH,high(nodevice_msg*2)
	ldi	ZL,low(nodevice_msg*2)
	rcall	sendline		; nodevice
	
	sec
	ret
	
detect_Device:
	sbrs	chipdata,7
	rjmp	detect_lowspeed
;;; hispeed device
	ldi	chipadr,CSOFcnt
	ldi	chipdata,0xAE
	rcall	USBWrite
	ldi	chipadr,DATAset
	ldi	chipdata,0xE0
	rcall	USBWrite
	ldi	chipadr,CtrlReg
	ldi	chipdata,0x05
	rcall	USBWrite
	ser	gr0
	sts	full_speed,gr0
	rjmp	fin_detect
detect_lowspeed:
	ldi	chipadr,CSOFcnt
	ldi	chipdata,0xEE
	rcall	USBWrite
	ldi	chipadr,DATAset
	ldi	chipdata,0xE0
	rcall	USBWrite
	ldi	chipadr,CtrlReg
	ldi	chipdata,0x21
	rcall	USBWrite
	clr	gr0
	sts	full_speed,gr0
	rjmp	fin_detect

fin_detect:
	ldi	chipadr,EP0Status
	ldi	chipdata,0x50
	rcall	USBWrite
	ldi	chipadr,EP0Counter
	ldi	chipdata,0
	rcall	USBWrite
	ldi	chipadr,EP0Control
	ldi	chipdata,1
	rcall	USBWrite
	rcall	wait10ms
	rcall	wait10ms
	rcall	wait10ms
	ldi	chipadr,IntStatus
	ldi	chipdata,0xff		; all clear
	rcall	USBWrite

	clc
	
	ret

; --------------------------------
; Hw Sof
; --------------------------------
hw_sof:
	ldi	chipadr,CtrlReg
	ldi	chipdata,0x08
	rcall	USBWrite	; reset
	rcall	wait10ms
	clr	chipdata
	rcall	USBWrite
	rcall	wait10ms
	ldi	chipadr,DATAset
	ldi	chipdata,0xe0
	rcall	USBWrite	; Setup SOF 1ms interval
	ldi	chipadr,cSOFCnt
	ldi	chipdata,0xae
	rcall	USBWrite
	ldi	chipadr,CtrlReg
	ldi	chipdata,1
	rcall	USBWrite
	ldi	chipadr,EP0Control
	ldi	chipdata,0x01
	rcall	USBWrite	; Start SOF/EOP
	
	ret

; ---------------------------------
; USB Reset
; ---------------------------------
USBReset:
	ldi	chipadr,CtrlReg
	rcall	USBRead
	mov	gr3,chipdata
	ori	chipdata,0x08		; setup usb reset
	rcall	USBWrite
	rcall	wait10ms
	rcall	wait10ms
	mov	chipdata,gr3
	rcall	USBWrite		; usb ebable
	ldi	gr2,25
	rcall	wait10xms		; wait 250ms
	ldi	chipadr,IntStatus
	ldi	chipdata,0xFF		;
	rcall	USBWrite

	ret

; ---------------------------------
; Errata patch 1
; in : chipdata
; ---------------------------------
erratta_patch_1:
	lds	gr0,full_speed
	cp	gr0,zero		
	breq	erratta_p1_1		; low speed
	mov	gr0,chipdata		;; backup
	ldi	chipadr,CSOFcnt
	rcall	USBRead
	lds	gr1,p1
	cp	chipdata,gr1
	brcc	erratta_p1_2
	ori	gr0,0x20
erratta_p1_2:
	mov	chipdata,gr0
erratta_p1_1:

	ret
	

;------------------------------------
; USB-A Done Interrupt wait
; gr1 : WAITTIME
;------------------------------------
IntUSBAWait:
	sts	timercnt,zero		; clear count(0.1sec freerun)
	ldi	chipadr,IntStatus
IntUSBAWait_loop:
	rcall	USBRead
	sbrc	chipdata,0
	rjmp	IntUSBAWait_fin
	mov	gr0,chipdata
	andi	gr0,0x20		; check remove/insert or Reset
	brne	IntUSBA_disconnect
	lds	gr0,timercnt
	cp	gr0,gr1			; 3sec wait
	brcs	IntUSBAWait_loop
	;;

	ldi	gr1,TIMEOUT1_ERR
	ser	gr0			; return -1
	
	ret				; timeout

IntUSBA_disconnect:
	ser	gr0
	sts	disconnected,gr0
	ldi	gr1,DISCONNECT_ERR
	
	ret

IntUSBAWait_fin:
	clr	gr0
	clr	gr1
	
	ret

; ---------------------------------------------
; usb Xfer
; gr0     = dev
; gr1     = ep
; gr2     = pid
; gr3     = iso
; gr4     = payload	(max 256)
; gr5     = len		(max 256)
; gr6 - 16 : broken
; Z       = *buf
; return
; gr1:gr0 = len
; gr1 == 0xff  : error
; ---------------------------------------------
usbxfer_disconnect1:
	ser	gr0
	sts	disconnected,gr0	;; disconnect
	ldi	gr1,DISCONNECT_ERR
	
	ret

USBXfer:
;;	clr	gr14		; DDD = 0
;;	clr	gr16		; rem = 0
	lds	gr6,disconnected
	or	gr6,zero
	brne	usbxfer_disconnect1
	ldi	chipadr,IntStatus	;; intr
	rcall	USBRead
	andi	chipdata,0x20
	brne	usbxfer_disconnect1
;
	sts	DDD,zero
	sts	rem,zero
	mov	gr7,gr1		; ep
	mov	gr6,gr0		; gr6 = dev
	mov	gr0,gr2		; gr2 = pid
	andi	gr0,0xf
	lsl	gr0
	lsl	gr0
	lsl	gr0
	lsl	gr0
	mov	chipdata,gr7	; ep
	or	chipdata,gr0	; npid
	ldi	chipadr,EP0Status
	rcall	USBWrite
	ldi	chipadr,EP0Counter
	mov	chipdata,gr6
	andi	chipdata,0x7f
	rcall	USBWrite	; dev
;;
	ldd	gr0,Y+DevSpeed
	cp	gr0,zero
	breq	usbxfer_hip1
	mov	gr0,gr4		;
	ldi	gr1,24
	add	gr0,gr1
	rjmp	usbxfer_p1set
usbxfer_hip1:
	mov	gr0,gr4
	lsr	gr0
	lsr	gr0
	lsr	gr0
	inc	gr0
	inc	gr0
	inc	gr0
usbxfer_p1set:
	sts	p1,gr0
	cpi	gr2,PID_IN
	brne	usbxfer_nopreamble1
	ldd	gr0,Y+DevSpeed
	cp	gr0,zero
	breq	usbxfer_nopreamble1
;; establish preamble packet
	ldi	chipadr,EP1Status
	ldi	chipdata,0xc0
	rcall	USBWrite
	ldi	chipadr,EP1XferLen
	ldi	chipdata,0
	rcall	USBWrite
	push	YL
	push	YH
	lds	gr0,HubDevNum
	rcall	GetBaseTblAdr
	ldd	chipdata,Y+DevAddr
	pop	YH
	pop	YL
	ldi	chipadr,EP1Counter
	rcall	USBWrite
;;
usbxfer_nopreamble1:
	cp	gr5,gr4		;len > payload
	brcc	set_nextbuf
;; 
;;	mov	gr15,gr5	;rLen = len
	sts	rLen,gr5
	rjmp	usbxfer_sendsize
set_nextbuf:
	ldi	gr0,cMemStart	; data0
	add	gr0,gr4		; + payload
	mov	gr6,gr0		; data1
;;	mov	gr15,gr4	; rLen = payload
	sts	rLen,gr4
usbxfer_sendsize:
	ldi	chipadr,EP0XferLen
;;	mov	chipdata,gr15	; rLen
	lds	chipdata,rLen	; rLen
	rcall	USBWrite	; EP0 xfersize
	ldi	chipadr,EP0Address
	ldi	chipdata,cMemStart ; data0
	rcall	USBWrite	; EP0 xferaddr
;;
	cpi	gr2,PID_SOF	; ? Send Software ARM?
	brne	usbxfer_notsof
	ldi	chipadr,EP0Control
	ldi	chipdata,1
	rcall	USBWrite	; arm & return
	ldi	chipadr,IntStatus
usbxfer_sofloop:
	rcall	USBRead
	mov	gr0,chipdata
	andi	gr0,0x20	; usb reset / insert/remove
	brne	usbxfer_disconnect
	and	chipdata,one
	breq	usbxfer_sofloop
	mov	gr0,gr5
	clr	gr1
	
	ret

usbxfer_disconnect:
	rjmp	usbxfer_disconnect1
;;
usbxfer_notsof:
	mov	gr1,gr3
	ldi	gr3,DATA0_RD
	ldd	gr0,Y+DevSpeed
	or	gr3,gr0		; Set Preamble
	or	gr1,zero	; iso check
	breq	PC+2
	ori	gr3,ISO_BIT	; gr3 = Cmd

	cpi	gr2,PID_IN
	breq	usbxfer_pidin
	ldi	chipadr,cMemStart ; data0
;;	mov	chipdata,gr15	  ; rLen
	lds	chipdata,rLen	  ; rLen
;; Z is readpointer
	push	ZH
	push	ZL
	rcall	USBBufWrite
	pop	ZL
	pop	ZH
	ldi	gr3,DATA0_WR
	ldd	gr0,Y+DevSpeed
	or	gr3,gr0

usbxfer_pidin:
	cpi	gr2,PID_SETUP
	breq	usbxfer_pidsetup

	cp	gr7,zero	; ep = 0 ?
	brne	episnotzero
	ldi	gr0,0x40
;;	mov	gr14,gr0	; DDD
	sts	DDD,gr0		; DDD
	rjmp	usbxfer_pidsetup
episnotzero:
	push	YH
	push	YL
	ldi	gr0,DToggle
	add	YL,gr0
	adc	YH,zero
	add	YL,gr7		; ep
	adc	YH,zero
;;	ld	gr14,Y		; DDD
	ld	gr0,Y		; DDD
	sts	DDD,gr0
	pop	YL
	pop	YH

usbxfer_pidsetup:
;;	clr	gr10		; time_out = 0
	sts	timeout,zero	; time_out = 0
	mov	tgr8,one	; retry = 1
	clr	tgr9		; tgr9:tgr8
	clr	tgr7
	mov	gr8,gr3		; 
;;	or	gr8,gr14	; wCMD = CMD | DDD
	lds	gr0,DDD
	or	gr8,gr0		; wCMD = CMD | DDD

;;---------------------------------------------------------
usbxfer_loop:

	cp	tgr8,zero
	brne	usbxfer_goretry
	cp	tgr9,zero
	brne	usbxfer_goretry
	cp	tgr7,zero
	breq	usbxfer_noretry
usbxfer_goretry:
	ldi	chipadr,IntStatus
	ldi	chipdata,INTMASK
	rcall	USBWrite
	
	cpi	gr2,PID_OUT
	breq	usbxfer_pidout
;;	mov	chipdata,gr14
	lds	chipdata,DDD
	or	chipdata,gr3		; CMD | DDD
	rjmp	usbxfer_check_sofc1
usbxfer_pidout:
	mov	chipdata,gr8		; wCmd
usbxfer_check_sofc1:
	cli
	rcall	erratta_patch_1		; for Erratta
	ldi	chipadr,EP0Control
	rcall	USBWrite
usbxfer_preamble1:
	cpi	gr2,PID_IN
	brne	usbxfer_noretry
	ldd	gr0,Y+DevSpeed
	cp	gr0,zero
	breq	usbxfer_noretry
	rcall	waitpre
	rcall	wait50us
;	rcall	wait50us
	ldi	chipadr,EP1Control
	ldi	chipdata,1
	rcall	USBWrite
	rcall	waitpre
;;
usbxfer_noretry:
	sei
	ldi	gr0,250
	cp	tgr8,gr0
	add	tgr8,one		; retry.l ++
	adc	tgr9,zero		; retry.h + carry
	adc	tgr7,zero		; retry.h + carry
	mov	gr0,gr5			; len
;;	sub	gr0,gr15		; len-rLen
	lds	gr1,rLen
	sub	gr0,gr1			; len-rLen
	cp	gr0,gr4			; gr0=len-rlen>=payload
	brcs	PC+2			; 
	;; cf = 0
	mov	gr0,gr4			; gr0 = payload
	ldi	gr1,cMemStart		; gr1 = data0
	sbrs	gr3,6			; bit6 on?
	mov	gr1,gr6			; gr1 = data1
;;
;;	mov	gr13,gr1		; addr
	sts	addr,gr1		; addr
;;	mov	gr12,gr0		; bLen
	sts	bLen,gr0		; bLen
;;
	cpi	gr2,PID_IN
	breq	usbxfer_pidin2

	mov	chipadr,gr1		; addr
	mov	chipdata,gr0		; bLen
	push	ZH
	push	ZL
;;	add	ZL,gr15			; buf+=rLen
	lds	gr0,rLen
	add	ZL,gr0			; buf+=rLen
	adc	ZH,zero			; next pos
	rcall	USBBufWrite
	pop	ZL
	pop	ZH
;;
usbxfer_pidin2:
	cpi	gr2,PID_OUT		; pid==PID_OUT?
	brne	usbxfer_pidout3
	cp	tgr9,zero
	brne	usbxfer_pidout3		; 
	cp	tgr7,zero
	brne	usbxfer_pidout3		; 
	ldi	gr0,3
	cp	tgr8,gr0		; retry<=2 ?
	brcc	usbxfer_pidout3		; 
;; 
;;	mov	chipadr,gr13		; addr
	lds	chipadr,addr		; addr
;;	mov	chipdata,gr12		; bLen
	lds	chipdata,bLen		; bLen
	push	ZH
	push	ZL
;;	add	ZL,gr15			; rLen
	lds	gr0,rLen
	add	ZL,gr0			; rLen
	adc	ZH,zero			; next pos
	rcall	USBBufWrite
	pop	ZL
	pop	ZH
;;
usbxfer_pidout3:
	ldi	gr1,10
	rcall	IntUSBAWait
	cp	gr0,zero
	breq	PC+2
	ret				; ERROR

;	ldi	gr0,'K'
;	rcall	senddata

usbxfer_loopfin:
	ldi	chipadr,EP0Status
	rcall	USBRead

;	mov	gr1,chipdata
;	rcall	dispresult

;;	mov	gr11,chipdata		; gr11 = result
	sts	result,chipdata		; gr11 = result

;	cpi	gr2,PID_IN
;	brne	usbxfer_chkresult0
;	ldd	gr0,Y+DevSpeed
;	cp	gr0,zero
;	breq	usbxfer_chkresult0
;	sbrc	chipdata,6		; result & 0x40 == 0?
;	rjmp	usbxfer_chkresult0
;;	ldi	gr0,'@'
;;	rcall	senddata	
;	push	chipdata
;	ldi	chipadr,IntStatus
;	ldi	chipdata,INTMASK
;	rcall	USBWrite
;	ldi	chipadr,EP0Control
;	ldi	chipdata,1
;	rcall	USBWrite
;	ldi	gr1,20
;	rcall	IntUSBAWait
;	pop	chipdata
;	cp	gr0,zero
;	breq	PC+2
;	ret
usbxfer_chkresult0:
	sbrs	chipdata,0		; chipdata & 1
	rjmp	usbxfer_nosetup
;; USB-A Interrupt
;	ldi	gr0,'K'
;	rcall	senddata
	ldi	gr0,0x40
	eor	gr8,gr0			; wCMD ^= 0x40
	clr	tgr8			; clear retry
	clr	tgr9
	clr	tgr7
	eor	gr3,gr0			; CMD ^= 0x40

	cpi	gr2,PID_SETUP
	breq	usbxfer_dddset

;;	eor	gr14,gr0		; DDD^=0x40
	lds	gr1,DDD
	eor	gr1,gr0			; DDD^=0x40
	sts	DDD,gr1
	cp	gr7,zero		; ep=gr7
	breq	usbxfer_dddset		; ep==0?
	
	push	YH
	push	YL
	ldi	gr0,DToggle
	add	YL,gr0
	adc	YH,zero
	add	YL,gr7
	adc	YH,zero
;;	st	Y,gr14			;gr14 = DDD
;;	lds	gr0,DDD			;gr14 = DDD
	st	Y,gr1			;gr14 = DDD
	pop	YL
	pop	YH

usbxfer_dddset:
	cpi	gr2,PID_IN
	brne	usbxfer_nopid2
	ldi	chipadr,EP0Counter
	rcall	USBRead
;;	mov	gr16,chipdata		; gr16 = rem
	sts	rem,chipdata		; gr16 = rem

usbxfer_nopid2:
;;	sub	gr5,gr15			; len-rlen
	lds	gr0,rLen
	sub	gr5,gr0				; len-rlen
	cp	gr5,zero
	breq	usbxfer_nosend
;;	cp	gr12,zero			; bLen
	lds	gr0,bLen
	cp	gr0,zero			; bLen
	breq	usbxfer_nosend

	ldi	chipadr,EP0XferLen
;;	mov	chipdata,gr12
	lds	chipdata,bLen
	rcall	USBWrite		; blen
	ldi	chipadr,EP0Address
;;	mov	chipdata,gr13
	lds	chipdata,addr
	rcall	USBWrite		; address
	ldi	chipadr,IntStatus
	ldi	chipdata,INTMASK
	rcall	USBWrite		; status clear
	cpi	gr2,PID_IN
	brne	usbxfer_next3
;;	cp	gr16,zero		; rem = 0 ?
	lds	gr0,rem
	cp	gr0,zero		; rem = 0 ?
	brne	usbxfer_next3
;;	mov	chipdata,gr14		; chipdata = DDD
	lds	chipdata,DDD		; chipdata = DDD
	or	chipdata,gr3		; Cmd | DDD
	cli				; !
	rcall	erratta_patch_1
	ldi	chipadr,EP0Control
	rcall	USBWrite
;;
	ldd	gr0,Y+DevSpeed
	cp	gr0,zero
	breq	usbxfer_nosend
	rcall	waitpre
	rcall	wait50us
;	rcall	wait50us
	ldi	chipadr,EP1Control	; preamble
	ldi	chipdata,1
	rcall	USBWrite
	rcall	waitpre
	rjmp	usbxfer_nosend
usbxfer_next3:
	cpi	gr2,PID_OUT
	brne	usbxfer_nosend
	mov	chipdata,gr8
	rcall	erratta_patch_1
	ldi	chipadr,EP0Control
	rcall	USBWrite		; wCmd
;;
usbxfer_nosend:
	sei				; 
	cpi	gr2,PID_IN
	brne	usbxfer_norecv
	ldi	chipadr,cMemStart	; data0
	sbrs	gr3,6			; cmd bit6
	mov	chipadr,gr6		; data1
;;	mov	chipdata,gr15		; rLen
	lds	chipdata,rLen		; rLen
	push	ZH
	push	ZL
	rcall	USBBufRead
	pop	ZL
	pop	ZH

;	ldi	gr0,'g'
;	rcall	senddata

;;	cp	gr16,zero		; rem
	lds	gr0,rem
	cp	gr0,zero		; rem
	breq	usbxfer_norecv

	mov	gr0,gr5			; short packet detect
	clr	gr1
	
	ret				;;
usbxfer_norecv:
	cp	gr5,zero		;; len == 0?
	brne	usbxfer_goloop
	rjmp	usbxfer_mainloopfin		;; success!
usbxfer_goloop:
;;	add	ZL,gr15
	lds	gr0,rLen
	add	ZL,gr0
	adc	ZH,zero			;; buf+=rLen
;;	mov	gr15,gr12		;; rLen = blen
	lds	gr0,bLen
	sts	rLen,gr0		;; rLen = blen

usbxfer_nosetup:
;;	mov	gr0,gr11
	lds	gr0,result
	sbrs	gr0,2			;; result & 4
	rjmp	usbxfer_notimeout
;;
;;	ldi	gr0,3
;;	cp	gr10,gr0
	lds	gr0,timeout
	cpi	gr0,5			;;
	brcs	usbxfer_undertimeout
;; timeout err
	ldi	gr1,TIMEOUT0_ERR
	ser	gr0			;; return -1

	ret

usbxfer_undertimeout:
;;	inc	gr10			;; timeout ++
	lds	gr0,timeout
	inc	gr0
	sts	timeout,gr0
	rjmp	usbxfer_incimeout
usbxfer_notimeout:
;;	clr	gr10			;; timeout = 0
	sts	timeout,zero
usbxfer_incimeout:

;;	mov	gr0,gr11
	lds	gr0,result
	sbrs	gr0,7			;; result & 0x80 (STALL)
	rjmp	usbxfer_nostall
;; stall
	mov	gr0,gr5
	clr	gr1			;; return len

	ret

usbxfer_nostall:
	lds	gr0,enablenak
	cp	gr0,zero
	breq	usbxfer_nonak
	lds	gr0,result
	sbrs	gr0,6
	rjmp	usbxfer_loop
;; NAK 
	ser	gr0
	ser	gr1
	ret

usbxfer_nonak:
	cp	tgr7,one
	brne	usbxfer_loop1
	ldi	gr0,high(MAX_RETRY)
	cp	tgr9,gr0
	brne	usbxfer_loop1
	ldi	gr0,low(MAX_RETRY)
	cp	tgr8,gr0
	breq	usbxfer_break
usbxfer_loop1:
	rjmp	usbxfer_loop
	

usbxfer_break:
	ldi	gr1,RETRY_ERR
	ser	gr0
	
	ret

usbxfer_mainloopfin:
;;	mov	gr0,gr11
	lds	gr0,result
	sbrs	gr0,0
	rjmp	usbxfer_retryerr
;; 
	mov	gr0,gr5
	clr	gr1

	ret

usbxfer_retryerr:
	ldi	gr1,RETRY_ERR
	ser	gr0

	ret

; ---------------------------------------------
; EP0Xfer
; dev : gr1:gr0
; payload : gr2
; Z : data address
; ---------------------------------------------
EP0Xfer:
	mov	gr4,gr2	; payload
	ldi	gr2,PID_SETUP
	clr	gr3	; no iso
	ldi	gr5,8	; 8byte
	push	gr0
	push	gr1
	push	gr2
	push	gr4	; backup payload
	push	ZH
	push	ZL
	ldi	ZL,low(SETUPBUF)
	ldi	ZH,high(SETUPBUF)
;	sts	enablenak,one
	rcall	USBXfer
;	sts	enablenak,zero
	mov	gr6,gr0
	mov	gr7,gr1
	pop	ZL
	pop	ZH
	pop	gr4
	pop	gr2
	pop	gr1
	pop	gr0
	
	cp	gr7,zero
	breq	EP0Xfer_next

;	push	gr0		;; @@debug
;	ldi	gr0,'Z'
;	rcall	senddata
;	pop	gr0
	mov	gr1,gr7

	ret		;; NG.

EP0Xfer_next:
;	push	gr0		;; @@debug
;	ldi	gr0,'V'
;	rcall	senddata
;	pop	gr0

	ldi	tgr0,PID_IN
	lds	gr5,wLength
	cp	gr5,zero
	breq	EP0Xfer_noxfer
	lds	gr3,bmRequest
	sbrs	gr3,7		; check direction bit
	rjmp	EP0Xfer_in
	ldi	tgr0,PID_OUT
	ldi	gr2,PID_IN
	ldi	gr3,0	; no iso
	push	gr1	; dev
	push	gr0	; dev
	rcall	USBXfer
	mov	gr6,gr0
	mov	gr7,gr1
	pop	gr0
	pop	gr1
	clr	gr4	; clear payload

	cp	gr7,zero
	breq	EP0Xfer_noxfer
	
;	push	gr0		; @@debug
;	ldi	gr0,'X'
;	rcall	senddata
;	pop	gr0

	rjmp	EP0Xfer_noxfer
EP0Xfer_in:
	ldi	gr2,PID_OUT
	ldi	gr3,0	; no iso
	push	gr1	; dev
	push	gr0	; dev
	push	gr4	; payload
	rcall	USBXfer
	mov	gr6,gr0
	mov	gr7,gr1
	pop	gr4
	pop	gr0
	pop	gr1
	
	cp	gr7,zero
	breq	EP0Xfer_noxfer
	
;	push	gr0		; @@debug
;	ldi	gr0,'Y'
;	rcall	senddata
;	pop	gr0

EP0Xfer_noxfer:
	;; gr0,1
	mov	gr2,tgr0	; 
	clr	gr3		; no iso
	;; gr4
	clr	gr5		; 
	clr	ZH
	clr	ZL		; NULL
	push	gr7		; error code
	rcall	USBXfer
	mov	gr0,gr1		; 
	pop	gr7

	mov	gr1,gr7

	ret

; ---------------------------------------------
; Set RTC for P/ECE for BulkXfer test.
; ---------------------------------------------
SetRTC_PIECE:
	lds	gr0,PIECEDEVNUM
	cpi	gr0,0xff
	brne	PC+2
	ret
	push	YL
	push	YH
	rcall	GetBaseTblAdr
;;	ldi	gr0,'@'		; @@debug
;;	rcall	senddata
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	ldi	gr0,15
	st	Z+,gr0
	ldi	gr0,0
	st	Z+,gr0
	ldi	gr0,low(2001)
	st	Z+,gr0
	ldi	gr0,high(2001)
	st	Z+,gr0
	ldi	gr0,2
	st	Z+,gr0
	ldi	gr0,4
	st	Z+,gr0
	ldi	gr0,10
	st	Z+,gr0
	ldi	gr0,13
	st	Z+,gr0
	ldi	gr0,24
	st	Z+,gr0
	ldi	gr0,56
	st	Z+,gr0
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	ldi	gr0,2	;;
	ldi	gr1,10
	ldi	gr2,PID_OUT
	rcall	BulkXfer
	pop	YH
	pop	YL
	
	ret

; ---------------------------------------------
; Write Keydata 
; gr1 : Keydata
; ---------------------------------------------
WriteKey_PIECE:
	lds	gr0,PIECEDEVNUM
	cpi	gr0,0xff
	brne	PC+2
	ret
	sts	STATUSCNT,zero	;; mouse fix.
	push	YL
	push	YH
	push	gr1
	rcall	GetBaseTblAdr
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	ldi	gr0,11	;; COMWrite
	st	Z,gr0
	ldi	gr0,2	;; ENDPOINT
	ldi	gr1,1
	ldi	gr2,PID_OUT
	rcall	BulkXfer
	pop	gr1
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	ldi	gr0,0x80
	st	Z+,gr0	;; KEY REQUEST
	st	Z+,gr1
	ldi	gr0,2	;; ENDPOINT
	ldi	gr1,8
	ldi	gr2,PID_OUT
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	rcall	BulkXfer
	pop	YH
	pop	YL
	
	ret

; ---------------------------------------------
; Write Mousedata 
; gr0,gr1,gr2 : mousedata
; ---------------------------------------------
WriteKey_PIECE_Mouse:
	lds	gr3,PIECEDEVNUM
	cpi	gr3,0xff
	brne	PC+2
	ret
	push	YL
	push	YH
	push	gr0
	push	gr1
	push	gr2
	mov	gr0,gr3
	rcall	GetBaseTblAdr
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	ldi	gr0,11	;; COMWrite
	st	Z,gr0
	ldi	gr0,2	;; ENDPOINT
	ldi	gr1,1
	ldi	gr2,PID_OUT
	rcall	BulkXfer
	pop	gr2
	pop	gr1
	pop	gr0
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	ldi	gr3,0x81
	st	Z+,gr3	;; MOUSE REQUEST
	st	Z+,gr0
	st	Z+,gr1
	st	Z+,gr2
	ldi	gr0,2	;; ENDPOINT
	ldi	gr1,8
	ldi	gr2,PID_OUT
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	rcall	BulkXfer
	pop	YH
	pop	YL
	
	ret


; ---------------------------------------------
; BulkXfer
; Y : Device
; gr1 : gr0: Endpoint Number
; length : gr1
; gr2 : PID_OUT : BulkOut
;       PID_IN  : BulkIn
; Z : data address
; ---------------------------------------------
BulkXfer:
	mov	gr3,gr2
	mov	gr5,gr1
	mov	gr1,gr0
	ldd	gr0,Y+DevAddr
	push	YL
	push	YH
	add	YL,gr1
	adc	YH,zero
	ldd	gr4,Y+DPayload	;; 
	pop	YH
	pop	YL
	cp	gr5,zero	;; length check
	brne	PC+2
	ret
	ldi	tgr0,PID_IN
	cpi	gr3,PID_OUT
	brne	PC+2
	rjmp	BulkXfer_in
	ldi	tgr0,PID_OUT
	ldi	gr2,PID_IN
	ldi	gr3,0	; no iso
	push	gr1	; dev
	push	gr0	; dev
	rcall	USBXfer
	mov	gr6,gr0
	mov	gr7,gr1
	pop	gr0
	pop	gr1
	clr	gr4	; clear payload

	cp	gr7,zero
	breq	BulkXfer_noxfer
	
;	push	gr0		; @@debug
;	ldi	gr0,'X'
;	rcall	senddata
;	pop	gr0

	ret

BulkXfer_in:
	ldi	gr2,PID_OUT
	ldi	gr3,0	; no iso
	push	gr1	; dev
	push	gr0	; dev
	push	gr4	; payload
	rcall	USBXfer
	mov	gr6,gr0
	mov	gr7,gr1
	pop	gr4
	pop	gr0
	pop	gr1
	
	cp	gr7,zero
	breq	BulkXfer_noxfer
	
;	push	gr0		;; @@debug
;	ldi	gr0,'Y'
;	rcall	senddata
;	pop	gr0

BulkXfer_noxfer:

	ret

; ---------------------------------------------
; Vendor Command
; gr0 : bReq
; gr1 : bCmd
; gr2 : wValueL
; gr3 : wValueH
; gr4 : wIndexL
; gr5 : wIndexH
; gr6 : wLenL
; Z : Transfer Addr
; ---------------------------------------------
VendorCmd:
	sts	bmRequest,gr0
	sts	bRequest,gr1
	sts	wValue,gr2
	sts	wValue+1,gr3
	sts	wIndex,gr4
	sts	wIndex+1,gr5
	mov	gr0,gr6
	sts	wLength,gr0
	clr	gr0
	sts	wLength+1,gr0

	ldd	gr0,Y+DevAddr
	clr	gr1		;; ENDPOINT 0
	ldd	gr2,Y+DPayload	;; END POINT
	rcall	EP0Xfer
	
	ret

; ---------------------------------------------
; Get Descripter
; gr1:gr0 wValue
; gr2 : Length
; Z : Buffer Address
; ---------------------------------------------
GetDesc:
	mov	gr6,gr2
	mov	gr2,gr0
	mov	gr3,gr1
	ldi	gr0,0x80
	ldi	gr1,GET_DESCRIPTOR
	clr	gr5
	clr	gr4
	rcall	VendorCmd
	
	ret

; ---------------------------------------------
; Set Address
; gr1:gr0 Address
; ---------------------------------------------
SetAddress:
	mov	gr2,gr0
	mov	gr3,gr1
	clr	gr6
	clr	gr0
	ldi	gr1,SET_ADDRESS
	clr	gr5
	clr	gr4
	clr	ZH
	clr	ZL
	rcall	VendorCmd
	
	ret

; ---------------------------------------------
; Set Configuration
; gr1:gr0 Value
; gr2 Len
; Z : Setting Data
; ---------------------------------------------
SetConfig:
	mov	gr6,gr2
	mov	gr2,gr0
	mov	gr3,gr1
	ldi	gr0,0x00
	ldi	gr1,SET_CONFIGURATION
	clr	gr5
	clr	gr4
	rcall	VendorCmd
	
	ret

; ---------------------------------------------
; SL811HS MEMORY TEST
; ERRCOUNT = gr2:gr1
; both gr2,gr1 are zero, it is ok.
; ---------------------------------------------
MemoryTest:
	clr	gr1			; clear error counter1
	clr	gr2			; clear error counter2

	ldi	chipadr,EP0Buf
memtest_writeloop:
	mov	chipdata,chipadr	; SET DATA=ADDRESS
	rcall	USBWrite
	add	chipadr,one
	brcc	memtest_writeloop

	ldi	chipadr,EP0Buf
memtest_readloop:
	rcall	USBRead			; XL=Memory Data
	cp	chipadr,chipdata
	breq	PC+3
	add	gr1,one			; ERROR ++
	adc	gr2,zero
	mov	chipdata,chipadr	; 
	com	chipdata		; Check Inverse Data
	rcall	USBWrite
	rcall	USBRead
	com	chipdata		; 
	cp	chipadr,chipdata	;
	breq	PC+3
	add	gr1,one			; ERROR ++
	adc	gr2,zero
	add	chipadr,one
	brcc	memtest_readloop

;; Auto Incriment write test
	ldi	chipadr,EP0Buf
	mov	chipdata,chipadr
	rcall	USBWrite		; First Write
	inc	chipadr			; Adr++
memtest_awriteloop:
	mov	chipdata,chipadr
	rcall	USBWriteA		; adr++ auto incriment
	add	chipadr,one
	brcc	memtest_awriteloop

;; Auto Incriment read test
	ldi	chipadr,EP0Buf
	rcall	USBRead			; first read
	cp	chipadr,chipdata
	breq	PC+3
	add	gr1,one			; 
	adc	gr2,zero
	inc	chipadr
memtest_areadloop:
	rcall	USBReadA
	cp	chipadr,chipdata
	breq	PC+3			;
	add	gr1,one			; 
	adc	gr2,zero
	add	chipadr,one
	brcc	memtest_areadloop
;;
;;	memory all clear
	ldi	chipadr,EP0Buf
	clr	chipdata
	rcall	USBWrite
	inc	chipadr
memclear_loop:
	rcall	USBWriteA
	add	chipadr,one
	brcc	memclear_loop

	ret

; ---------------------------------------------
; Get Hub Descripter
; gr0 : Length
; Z   : Buffer Address
; ---------------------------------------------
GetHubDesc:
	mov	gr6,gr0		;; Length is 9byte
	clr	gr2
	ldi	gr3,0x29	;; Descriptor type is 0x29
	ldi	gr0,GET_HUB_DESCRIPTOR
	ldi	gr1,GET_DESCRIPTOR
	clr	gr5
	clr	gr4
	rcall	VendorCmd
	
	ret

; --------------------------------------------
; Set Port Feauture
; gr0 : Function
; gr1 : Port
; --------------------------------------------
SetPortFeature:
	mov	gr2,gr0
	clr	gr3
	mov	gr4,gr1
	clr	gr5
	ldi	gr0,SET_PORT_FEATURE
	ldi	gr1,SET_FEATURE
	clr	gr6
	rcall	VendorCmd
	
	ret

; --------------------------------------------
; Clear Port Feauture
; gr0 : Function
; gr1 : Port
; --------------------------------------------
ClearPortFeature:
	mov	gr2,gr0
	clr	gr3
	mov	gr4,gr1
	clr	gr5
	ldi	gr0,CLEAR_PORT_FEATURE
	ldi	gr1,CLEAR_FEATURE
	clr	gr6
	rcall	VendorCmd
	
	ret

; --------------------------------------------
; Get Port Status
; gr0 : Port
; Z : Buffer Address
; --------------------------------------------
GetPortStatus:
	ldi	gr2,4
	mov	gr6,gr2
	clr	gr2
	clr	gr3
	mov	gr4,gr0
	clr	gr5
	ldi	gr0,GET_PORT_STATUS
	ldi	gr1,GET_STATUS
	rcall	VendorCmd
	
	ret

; -------------------------------------------
; HID Get Descriptor
; gr0 : Value L
; gr1 : Value H
; gr2 : Length
; Z : Buffer Address
; -------------------------------------------
Hid_GetDescriptor:
	mov	gr6,gr2
	mov	gr2,gr0
	mov	gr3,gr1
	clr	gr4
	clr	gr5
	ldi	gr0,0x81
	ldi	gr1,GET_DESCRIPTOR
	rcall	VendorCmd
	
	ret

; -------------------------------------------
; Get Report
; gr0 : Report Type
; gr1 : Length
; Z : Buffer Address
; -------------------------------------------
GetReport:
	mov	gr6,gr1
	mov	gr2,gr0
	clr	gr3
	clr	gr4
	clr	gr5
	ldi	gr0,0xa1
	ldi	gr1,GET_REPORT
	rcall	VendorCmd
	
	ret

; -------------------------------------------
; Set Report
; gr0 : Report Type
; gr1 : Interface
; -------------------------------------------
SetReport:
	clr	gr6
	mov	gr2,gr0
	clr	gr3
	mov	gr4,gr1
	clr	gr5
	ldi	gr0,0x00
	ldi	gr1,SET_REPORT
	rcall	VendorCmd
	
	ret


; -------------------------------------------
; Set Report
; gr0 : Report ID
; gr1 : Report Type
; gr2 : len
; Z : Buffer
; -------------------------------------------
HidSetReport:
	mov	gr6,gr2
	mov	gr2,gr0
	mov	gr3,gr1
	clr	gr4
	clr	gr5
	ldi	gr0,0x21
	ldi	gr1,SET_REPORT
	rcall	VendorCmd
	
	ret

; -------------------------------------------
; Set Idle
; gr0 : ValueL
; gr1 : ValueH
; -------------------------------------------
SetIdle:
	clr	gr6
	mov	gr2,gr0
	mov	gr3,gr1
	clr	gr4
	clr	gr5
	ldi	gr0,0x21
	ldi	gr1,SET_IDLE
	rcall	VendorCmd
	
	ret

; -------------------------------------------
; Set Protocol
; gr0 : Protocol Type
; gr1 : Interface
; -------------------------------------------
SetProtocol:
	clr	gr6
	mov	gr2,gr0
	clr	gr3
	mov	gr4,gr1
	clr	gr5
	ldi	gr0,0x21
	ldi	gr1,SET_PROTOCOL
	rcall	VendorCmd
	
	ret

; -----------------------------------------
; Get Desc Probe
; ----------------------------------------
GetDescProbe:
;;	clr	gr0
	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	push	ZH
	push	ZL
	rcall	GetDesc
	pop	ZL
	pop	ZH

	ret

; ----------------------------------------
; Execute Key Read
; Y : Device Table Address
; gr0 : out  keydata (0xFF is nodata)
; ----------------------------------------
ExecuteKeyRead:
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	ldi	gr5,8		; for Boot Protocol
	sts	enablenak,one
	rcall	ReadData
	sts	enablenak,zero
	cp	gr1,zero
	brne	ExecKeyRead_nokey

	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	rcall	ConvKeyCode	; usb key -> ascii
	ldi	ZL,low(PrevKey)
	ldi	ZH,high(PrevKey)
	ld	gr1,Y		; Device Number
	add	ZL,gr1
	adc	ZH,zero
	ld	gr1,Z		; previous key
	cp	gr0,gr1
	breq	ExecKeyRead_nokey
	st	Z,gr0		; set new key
	cpi	gr0,0xff
	brne	PC+2
	ret
;; special control code?
	cpi	gr0,0xf0
	brcc	PC+2
	ret			; fin.
;; LED CONTROL
	cpi	gr0,0xfe	; _CAPSLOCK
	brne	PC+2
	ldi	gr1,0x02
	cpi	gr0,0xfd	; _NUMLOCK
	brne	PC+2
	ldi	gr1,0x01
	cpi	gr0,0xfc	; _SCROLLLOCK
	brne	PC+2
	ldi	gr1,0x04
	lds	gr0,KEYSTATUS
	eor	gr0,gr1
	sts	KEYSTATUS,gr0
	rcall	SetLEDStatus
	ser	gr0

	ret

ExecKeyRead_nokey:
	ser	gr0
	
	ret

; ----------------------------------------
; Execute Mouse Read
; Y : Device Table Address
; gr0 : button (0xff) out  x addr
; gr1 : x
; gr2 : y
; 20times/per 1 out
; ----------------------------------------
ExecuteMouseRead:
	lds	gr3,STATUSCNT
	inc	gr3
	sts	STATUSCNT,gr3
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	ldi	gr5,3		; for Boot Protocol
	sts	enablenak,one
	rcall	ReadData
	sts	enablenak,zero
	cp	gr0,zero
	breq	PC+2
	rjmp	ExecMouseRead_nodata_exit
	cp	gr1,zero
	breq	PC+2
	rjmp	ExecMouseRead_nodata_exit

	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	ld	gr0,Z+
	ld	gr1,Z+
	ld	gr2,Z+
	ldd	gr3,Y+DevNum
	ldi	XL,low(PrevButton)
	ldi	XH,high(PrevButton)
	mov	gr4,gr3
	lsl	gr4
	add	gr4,gr3
	add	XL,gr4
	adc	XH,zero
	push	XL
	push	XH
	ld	gr3,X+
	cp	gr0,gr3
	brne	ExecMouseRead_ok1
	ld	gr3,X+
	cp	gr3,gr1
	brne	ExecMouseRead_ok
	ld	gr3,X+
	cp	gr3,gr2
	brne	ExecMouseRead_ok
	rjmp	ExecMouseRead_ng
ExecMouseRead_ok:
	or	gr2,gr2
	brne	ExecMouseRead_ok1
	or	gr1,gr1
	brne	ExecMouseRead_ok1
	rjmp	ExecMouseRead_ng
ExecMouseRead_ok1:
	pop	XH
	pop	XL
	st	X+,gr0
	st	X+,gr1
	st	X+,gr2
	lds	gr3,STATUSCNT
	cpi	gr3,INTERVAL_COUNT
	brcs	ExecMouseRead_ngx
	sts	STATUSCNT,zero
	clc

	ret

;; buttonω͖M
ExecMouseRead_ok2:
	pop	XH
	pop	XL
	st	X+,gr0
	st	X+,gr1
	st	X+,gr2
	sts	STATUSCNT,zero
	clc

	ret


ExecMouseRead_ng:
	pop	XH
	pop	XL
	st	X+,gr0
	st	X+,gr1
	st	X+,gr2
ExecMouseRead_ngx:
	sec

	ret

ExecMouseRead_nodata:
	pop	XH
	pop	XL
ExecMouseRead_nodata_exit:
	sec
	
	ret


; -------------------------------------------
; Read Hub Data
; Y : Device Table Address
; Z : Read Buffer Address
; gr1 : 0 : ok, else err
; gr0 : status
; -------------------------------------------
ReadHubData:
	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	ldd	gr0,Y+DevAddr
	ldd	gr1,Y+IntrEP
	ldi	gr2,PID_IN
	push	YL
	push	YH
	add	YL,gr1
	adc	YH,zero
	ldd	gr4,Y+DPayload
	pop	YH
	pop	YL
	clr	gr3
	ldi	gr5,1	;; for 4-8ports
	push	ZL
	push	ZH
	sts	enablenak,one
	rcall	USBXfer
	sts	enablenak,zero
	pop	ZH
	pop	ZL
	cp	gr1,zero
	brne	ReadHubData_nak
	cp	gr0,zero
	brne	ReadHubData_nak
	ld	gr0,Z
	ret

ReadHubData_nak:
	clr	gr0

	ret


; -----------------------------------------
; Hub Port Power On
; -----------------------------------------
PowerOnHubPort:
	;; Hub VBUS On
	ldi	gr0,PORT_POWER
	push	gr1		;; !!!
	rcall	SetPortFeature

;; for Power On Good

	lds	gr2,HubWait
	cp	gr2,zero
	breq	PoweronHubPort_nowait

PoweronHubPort_loop:
	rcall	wait1ms
	rcall	wait1ms
	dec	gr2
	brne	PoweronHubPort_loop

PoweronHubPort_nowait:
	rcall	wait10ms
	rcall	wait10ms
;	ldi	gr2,100
;	rcall	wait10xms
	pop	gr1		;; !!!

	ret

; ----------------------------------------------
;  Enable Selected Hub port
;  gr1 : Port Number
; ----------------------------------------------
EnableHubPort:
	;; Hub Port Reset
	ldi	gr0,PORT_RESET
	push	gr1
	rcall	SetPortFeature
	rcall	wait10ms
	rcall	wait10ms
	rcall	wait10ms
;	rcall	wait10ms
;	rcall	wait10ms
	pop	gr1

	;; Hub Port Enable
;	ldi	gr0,PORT_ENABLE
;	push	gr1
;	rcall	SetPortFeature
;	;; Wait Device Dependent
;	rcall	wait10ms
;	rcall	wait10ms
;	rcall	wait10ms
;	rcall	wait10ms
;	rcall	wait10ms
;	pop	gr1

	ret

; ----------------------------------------------
; Get Hub Port Status
; ----------------------------------------------
GetHubPortStatus:
	push	gr1
	mov	gr0,gr1
	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	rcall	GetPortStatus
	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	pop	gr1

	ret

; ----------------------------------------------
;  Disable Selected Hub port
;  gr1 : Port Number
; ----------------------------------------------
DisableHubPort:
	push	YH
	push	YL
	lds	gr0,HubDevNum
	rcall	GetBaseTblAdr	; Y = Table Address
	;; Hub Port Disable
	ldi	gr0,PORT_ENABLE
	push	gr1
	rcall	ClearPortFeature
	pop	gr1
	pop	YL
	pop	YH

	ret

; ----------------------------------------------
;  Clear All Change Bit
;  gr0 : Status
;  gr1 : Port Number
; ----------------------------------------------
ClearAllChangebit:

	push	YH
	push	YL
	push	gr0
	lds	gr0,HubDevNum
	rcall	GetBaseTblAdr	; Y = Table Address
	pop	gr0
	sbrs	gr0,0
	rjmp	ClearAllChangebit_1
	push	gr0
	ldi	gr0,C_PORT_CONNECTION
	push	gr1
	rcall	ClearPortFeature
	pop	gr1
	pop	gr0
ClearAllChangebit_1:
	sbrs	gr0,1
	rjmp	ClearAllChangebit_2
	push	gr0
	ldi	gr0,C_PORT_ENABLE
	push	gr1
	rcall	ClearPortFeature
	pop	gr1
	pop	gr0
	
ClearAllChangebit_2:
	sbrs	gr0,2
	rjmp	ClearAllChangebit_3
	push	gr0
	ldi	gr0,C_PORT_SUSPEND
	push	gr1
	rcall	ClearPortFeature
	pop	gr1
	pop	gr0

ClearAllChangebit_3:
	sbrs	gr0,3
	rjmp	ClearAllChangebit_4
	push	gr0
	ldi	gr0,C_PORT_OVER_CURRENT
	push	gr1
	rcall	ClearPortFeature
	pop	gr1
	pop	gr0

ClearAllChangebit_4:
	sbrs	gr0,4
	rjmp	ClearAllChangebit_5
	push	gr0
	ldi	gr0,C_PORT_RESET
	push	gr1
	rcall	ClearPortFeature
	pop	gr1
	pop	gr0

ClearAllChangebit_5:

	pop	YL
	pop	YH

	ret



; ----------------------------------------------------
; Query USB Device at selected Hub Port
; gr1 : Port Number
; ----------------------------------------------------
QueryUSBDevice:
	push	gr1
	lds	gr0,HubDevNum
	rcall	GetBaseTblAdr	;; get hub device table
;	push	gr1
;	ldi	gr2,100
;	rcall	wait10xms
;	pop	gr1
	rcall	EnableHubPort
;;
	rcall	GetHubPortStatus

	ldi	gr0,0x10
	rcall	dump

	ldd	gr0,Z+2
	push	ZH
	push	ZL
	rcall	ClearAllChangeBit
	pop	ZL
	pop	ZH
	
	ld	gr0,Z+		;; Enabled?
	andi	gr0,0x02
	breq	QueryUSBDev_nodev
;;
	clr	gr3
	ld	gr0,Z		;; LowSpeedDevice?
	andi	gr0,0x02
	breq	QueryUSBDev_hispeed
	inc	gr3
;;
QueryUSBDev_hispeed:
	mov	gr0,gr3		;; set speed select
	rcall	FindNewUsbDevice
	cp	gr0,zero
	breq	QueryUSBDev_found
;;
	;; select port disabled
	rcall	DisableHubPort	; Port Disanle
QueryUSBDev_nodev:
	mov	gr0,gr1
	rcall	ClearHubPortStat

	pop	gr1
	ser	gr0
	ret

QueryUSBDev_found:
	;; Y : Device Table Addr
	std	Y+DHubPorts,gr1	; No HUB Device is port num
	mov	gr0,gr1
	rcall	SetHubPortStat
	pop	gr1
	clr	gr0
	
	ret

; ----------------------------------------------
;  Find ALL USB Device (1 Hub(4ports)) Only
; ----------------------------------------------
FindUSBDevice_Wait:
	ldi	gr2,255
	rcall	wait10xms
;;	ldi	gr0,'R'		;; @@debug
;;	rcall	senddata
	rjmp	FindUSBDevices2
FindUSBDevices:
	rcall	USBReset	;; ALL BUS RESET
FindUSBDevices2:
	clr	gr0
	rcall	FindNewUSBDevice
	;; Y is Device Base Address
	cp	gr0,zero
	breq	PC+2
	rjmp	FindUSBDevice_Fail
	;;
	ldd	gr0,Y+DevType
	cpi	gr0,HUB_DEVICE	;; if Root Device is not HUB,fin.
	breq	FindUSBDevice_HubFound
	rjmp	FindUSBDevice_Success

FindUSBDevice_HubFound:
	sts	HubDevNum,tgr1	;; Set Device Number

	;; Get Hub descriptor
	ldi	gr0,9		;; Descriptor Length
	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	push	ZH
	push	ZL
	rcall	GetHubDesc
	pop	ZL
	pop	ZH

	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	ldi	gr0,0x40
	rcall	dump

	;; Check Power on Wait
	ldd	gr0,Z+hub_bPwrOn2PwrGood	; gr0 * 2msec
	sts	HubWait,gr0

	;; Get Ports
	ldd	gr0,Z+hub_bNbrPorts
	cpi	gr0,MAX_HUBPORTS+1
	brcs	PC+2
	ldi	gr0,MAX_HUBPORTS
	cp	gr0,zero
	brne	PC+2
	ldi	gr0,MAX_HUBPORTS		; if 0 then MAX_PORTS
	sts	HubPortNum,gr0

	ldi	gr1,1		;; First Port
	ldi	gr2,1		;; st bit
FindUSBDev_Loop:
	lds	gr0,HubDevNum
	rcall	GetBaseTblAdr	; Y = Table Address
	rcall	PowerOnHubPort
	rcall	QueryUSBDevice
	inc	gr1
	lds	gr0,HubPortNum
	inc	gr0
	cp	gr1,gr0
	brcs	FindUSBDev_Loop

FindUSBDevice_Success:
	clr	gr0

;	ldi	ZH,high(HubWait)
;	ldi	ZL,low(HubWait)
;	ldi	gr0,0x100
;	rcall	dump

	ret

;--------------------------------------------
; Get Device Table Base Address
; gr0 : Device Number
;--------------------------------------------
GetBaseTblAdr:
	ldi	YH,high(DEVICE_TABLE)
	ldi	YL,low(DEVICE_TABLE)
	cpi	gr0,0
	breq	GetBaseTblAdr_set
GetBaseTblAdr_loop:
	adiw	YL,DEV_TABLE_SIZE
	dec	gr0
	brne	GetBaseTblAdr_loop
GetBaseTblAdr_set:

	ret

;--------------------------------------------
; Get Empty Table
; gr0 : Result (Index)
;--------------------------------------------
GetEmptyTable:
	push	gr2
	clr	gr2
GetEmptyTable_loop:
	mov	gr0,gr2
	rcall	GetBaseTblAdr
	ldd	gr0,Y+DevType
	cp	gr0,zero
	breq	GetEmptyTable_found
	inc	gr2
	cpi	gr2,MAX_DEVICES
	brne	GetEmptyTable_loop
	ser	gr0
	pop	gr2
	ret

GetEmptyTable_found:
	mov	gr0,gr2
	pop	gr2
	
	ret

;--------------------------------------------
; Clear Information
; tgr1 : DeviceNumber
; Y : Buffer Start Address
;--------------------------------------------
ClearDeviceInfo:
	push	gr0
	
;	ser	gr0
;	std	Y+devType,gr0	; Device Type  = 0xff is No device
	push	YH
	push	YL
	ldi	gr0,DEV_TABLE_SIZE
clearinfo_loop:
	st	Y+,zero
	dec	gr0
	brne	clearinfo_loop
	pop	YL
	pop	YH
	ldi	gr0,64		; 
	std	Y+DPayload,gr0	; Initialize EP0 Payload = 8
	std	Y+DevNum,tgr1	; Set Device Number
	cp	gr2,zero
	breq	ClearDeviceinfo_hispped
	ldi	gr0,PREAMBLE
	std	Y+DevSpeed,gr0
ClearDeviceinfo_hispped:
	
	pop	gr0
	
	ret

; -------------------------------------------------
; Set Endpoint Information
; -------------------------------------------------
SetEndpointInfo:
	push	YH			;; push Y
	push	YL
	ldd	gr0,Z+ed_bEPAdd
	mov	gr6,YH
	mov	gr7,YL
	mov	gr8,gr0			;; backup : ep->gr8
	mov	gr1,gr0
	andi	gr0,0x0f
	add	YL,gr0
	adc	YH,zero			;; ep offset added
	std	Y+DevEP,gr1		;; set EP
	ldd	gr1,Z+ed_bAttr
	std	Y+DAttr,gr1		;; Set Attribute
	andi	gr1,0x3
	cpi	gr1,0x3			;; Interrupt?
	brne	SetEndpointInfo_nointep
	sbrs	gr8,7			;; Input Endpoint?
	rjmp	SetEndpointInfo_nointep
	push	YH
	push	YL
	mov	YH,gr6
	mov	YL,gr7
	std	Y+IntrEP,gr0		;; Set Interrupt Ep
	pop	YL
	pop	YH
SetEndpointInfo_nointep:
	ldd	gr1,Z+ed_bInterval
	std	Y+DInterval,gr1		;; Set Interval
	ldd	gr0,Z+ed_wPayload+1
	andi	gr0,0x07		;; mask
	cp	gr0,zero
	ldi	gr0,MAX_PAYLOADSIZE
	brne	SetEndpoint_size
	ldd	gr0,Z+ed_wPayload
	cp	gr0,zero
	breq	SetEndpoint_defsize	;; if Payload = 0 then 8
	cpi	gr0,MAX_PAYLOADSIZE
	brcs	SetEndpoint_size
	ldi	gr0,MAX_PAYLOADSIZE	;; = 64
	rjmp	SetEndpoint_size
SetEndpoint_defsize:
	ldi	gr0,DEF_PAYLOADSIZE	;; = 8
SetEndpoint_size:
	std	Y+DPayload,gr0		;; Set Payload
	pop	YL
	pop	YH			;; restore Y

	ret

; -------------------------------------------------
; Set Device informations
; -------------------------------------------------
SetDeviceInfo:
	clr	gr4			;; interface found
	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	ldd	gr3,Z+cd_bNumIntf	;; Interface Number
	ld	gr0,Z
	add	ZL,gr0
	adc	ZH,zero			;; Interface Desc
SetDevInfo_mainloop:
	


	ldd	gr0,Z+id_bType
	cpi	gr0,DEV_INTERFACE
	breq	PC+2
	rjmp	SetDevInfo_Fail 	;; illegal device
	ldd	gr2,Z+id_bEndPoints	;; gr2 = end points
	cpi	gr2,ENDPOINTS
	brcs	PC+2			;; (fix ENDPOINT)
	ldi	gr2,ENDPOINTS-1		;; over endpoint count device
	ldd	gr0,Z+id_iClass		;; Device Class Checking
	cpi	gr0,HID_DEVICE
	breq	SetDevinfo_hiddev
	cpi	gr0,HUB_DEVICE
	breq	SetDevinfo_hubdev
	ldd	gr0,Y+DevType
	cpi	gr0,UNIQUEDEV_STARTINDEX	;; !
	brcs	SetDevInfo_SkipInterfaces	; Skip Interface
;; UNIQUE DEVICE
	ld	gr1,Z				; get interface length
	rjmp	SetEPDesc			;
SetDevinfo_hubdev:
;; HUB DEVICE CHECK
	dec	gr3			;; Interface --
	std	Y+DevType,gr0		;; Set Device Type
	ldd	gr0,Z+id_iSub
	std	Y+DevSubType,gr0	;; Set Device Sub Type
	ld	gr1,Z			;; get interface length
	rjmp	SetEPDesc
;; HID DEVICE CHECK
SetDevinfo_hiddev:
	dec	gr3			;; Interface --
	std	Y+DevType,gr0		;; Set Device Type
	ldd	gr1,Z+id_iSub
	andi	gr1,0x0f
	ldd	gr0,Z+id_iProto
	andi	gr0,0x0f
	lsl	gr0
	lsl	gr0
	lsl	gr0
	lsl	gr0
	or	gr0,gr1
	std	Y+DevSubType,gr0	;; Set Device Sub Type
;; Next HID Descriptor
	dec	gr3			;; interfaces --
	ld	gr1,Z			;;
	add	ZL,gr1
	adc	ZH,zero			;; HID
;;
;; if hid desc checked, ...
;;
SetEPDesc:
	ld	gr0,Z			;; 
	add	ZL,gr0
	adc	ZH,zero
	ldd	gr0,Z+1			;; get desc type
	cpi	gr0,DEV_ENDPOINT
	brne	SetEPDesc		;; Find Endpoint loop
	ser	gr4			;; interface found
SetEPDesc_loop:
	rcall	SetEndpointInfo		;; Set Endpoint Information
	ld	gr1,Z
	add	ZL,gr1
	adc	ZH,zero
	dec	gr2
	brne	SetEPDesc_loop
	rjmp	SetNextInterface

SetDevInfo_SkipInterfaces:
	dec	gr3
	ld	gr0,Z			;; next interface/desc
	add	ZL,gr0
	adc	ZH,zero
	ldd	gr0,Z+1			;; get desc type
	cpi	gr0,DEV_ENDPOINT
	brne	SetDevInfo_SkipInterfaces
	;;
SetDevInfo_SkipEndpoints:
	ld	gr0,Z			;; next endpoint/desc
	add	ZL,gr0
	adc	ZH,zero
	dec	gr2
	brne	SetDevInfo_SkipEndpoints
;;
SetNextInterface:
	cp	gr4,zero
	brne	SetDevInfo_Success
	dec	gr3
	cp	gr3,zero
	breq	SetDevinfo_Fail
	cpi	gr3,0x80
	brcc	PC+2
	rjmp	SetDevinfo_mainloop
SetDevinfo_Fail:

	ser	gr0
	ret

SetDevInfo_Success:
	clr	gr0
	
	ret

; ---------------------------------------
; Display Detect New Device Message
; Z:DescriptorBuf : DeviceDescriptor
; ---------------------------------------
DispDetectMessage:
	rcall	sendcrlf
	ldi	ZL,low(manufacture_msg*2)
	ldi	ZH,high(manufacture_msg*2)
	rcall	sendline
;;
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	ldd	gr0,Z+dd_iManufacturer
	ldd	tgr6,Z+dd_iProduct
	ldd	gr0,Z+dd_iManufacturer
	cp	gr0,zero
	breq	DispDetectMessage_nomanu
	ldi	gr2,0x7f 	; 
	ldi	gr1,DEV_STRING
	rcall	GetDescProbe
	cp	gr1,zero
	brne	DispDetectMessage_nomanu
	ld	gr0,Z+		; length
	cpi	gr0,3
	brcs	DispDetectMessage_nomanu
	ld	gr1,Z+		; type
	cpi	gr1,DEV_STRING
	brne	DispDetectMessage_nomanu
	;; gr0 = count
	subi	gr0,2
	rcall	sendline_ram2
	rcall	sendcrlf
	rjmp	DispDetectMessage_product
DispDetectMessage_nomanu:
	ldi	ZL,low(noname_msg*2)
	ldi	ZH,high(noname_msg*2)
	rcall	sendline

DispDetectMessage_product:
	ldi	ZL,low(product_msg*2)
	ldi	ZH,high(product_msg*2)
	rcall	sendline

	mov	gr0,tgr6
	cp	gr0,zero
	breq	DispDetectMessage_noproduct
	ldi	gr2,0x7f 	; 
	ldi	gr1,DEV_STRING
	rcall	GetDescProbe
	cp	gr1,zero
	brne	DispDetectMessage_noproduct
	ld	gr0,Z+		; length
	cpi	gr0,3
	brcs	DispDetectMessage_nomanu
	ld	gr1,Z+		; type
	cpi	gr1,DEV_STRING
	brne	DispDetectMessage_noproduct
	;; gr0 = count
	subi	gr0,2
	rcall	sendline_ram2
	rcall	sendcrlf

	ret

DispDetectMessage_noproduct:
	ldi	ZL,low(noname_msg*2)
	ldi	ZH,high(noname_msg*2)
	rcall	sendline

	ret

; ---------------------------------
; Send RAM Message
; Z : string address
; ---------------------------------
sendline_ram:
	ld	gr0,Z+
	or	gr0,gr0
	breq	sendlineram_fin
	rcall	senddata
	rjmp	sendline_ram
sendlineram_fin:
	ret

; ---------------------------------
; Send RAM Message 2
; gr0 : count
; Z : string address
; ---------------------------------
sendline_ram2:
	mov	gr1,gr0
sendline_ram2_loop:
	ld	gr0,Z+
	rcall	senddata
	dec	gr1
	breq	sendlineram2_fin
	ld	gr0,Z+	;; dummy read
	dec	gr1
	brne	sendline_ram2_loop
sendlineram2_fin:
	ret

;--------------------------------------------------
; Check Unique Device (P/ECE & USBWR Device Check)
; out: gr0  0xFF not found
;      else      devtype index
;--------------------------------------------------
CheckUniqueDevice:
	ldi	XL,low(DescriptorBuf)
	ldi	XH,high(DescriptorBuf)
	adiw	XL,dd_idVendor
	ldi	ZL,low(devid_table*2)
	ldi	ZH,high(devid_table*2)
	clr	gr1
CheckUniqueDevice_loop0:
	push	XL
	push	XH
	ldi	gr3,4	;; MID(W)+PID(W)
	clr	gr2
CheckUniqueDevice_loop1:
	lpm
	ld	gr0,X+
	cp	gr0,r0
	brne	PC+2
	inc	gr2
	add	ZL,one
	adc	ZH,zero
	dec	gr3
	brne	CheckUniqueDevice_loop1
	pop	XH
	pop	XL
	cpi	gr2,4
	breq	CheckUniqueDevice_Found
	inc	gr1
	cpi	gr1,DEVTABLE_COUNT
	brne	CheckUniqueDevice_loop0
	ser	gr0	;; not found

	ret

CheckUniqueDevice_Found:
	mov	gr0,gr1
	
	ret
	
;--------------------------------------------
; Find New USB Device
; gr0 : 0     Success
;       0xff  Fail
;--------------------------------------------
FindNewUSBDevice:
	mov	gr2,gr0		;
	rcall	GetEmptyTable
	cpi	gr0,0xff
	brne	PC+2
	ret			; NG (gr0 = 0xff)
	push	gr1		; !!
	mov	tgr1,gr0	; Device number(!)
	rcall	GetBaseTblAdr	; Y = Table Address

	; Clear Target Table
	; gr2 : speed select
	rcall	ClearDeviceInfo

	ldi	gr0,5
	mov	tgr3,gr0
FindUSBDevice_devloop:
	clr	gr0
	ldi	gr2,0x8 	; Default Payload is 8
	ldi	gr1,DEV_DEVICE
	rcall	GetDescProbe
	cp	gr1,zero
	breq	FindUSBDevice_Next1
	push	gr2
	ldi	gr2,200
	rcall	wait10xms
	pop	gr2
	dec	tgr3
	brne	FindUSBDevice_devloop
	rjmp	FindUSBDevice_Fail

FindUSBDevice_Next1:
	ldd	gr0,Z+dd_bMaxPacketSize0 ; Set Max Payload Size
	or	gr0,zero
	brne	PC+2
	ldi	gr0,8
	std	Y+DPayload,gr0
	rjmp	FindUSBDevice_Next

FindUSBDevice_Fail:

	pop	gr1
	ser	gr0
	ret

; ---------------------------------
FindUSBDevice_Next:
	clr	gr0
	ldi	gr2,0x12 	; 
	ldi	gr1,DEV_DEVICE
	rcall	GetDescProbe
	cp	gr1,zero
	brne	FindUSBDevice_Fail

	ldd	gr0,Z+dd_bNumConfigurations
	cp	gr0,one			 ; Configuration 1 only
	brne	FindUSBDevice_Fail

	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	ldi	gr0,0x40
	rcall	dump

	ldi	gr0,3
	mov	tgr3,gr0
FindUSBDevice_setadrloop:
	lds	gr0,DEVADDR_CNT		;; New
	clr	gr1
	rcall	SetAddress
	cp	gr0,zero
	breq	FindUSBDevice_setadrok
	dec	tgr3
	brne	FindUSBDevice_setadrloop
	rjmp	FindUSBDevice_Fail
	
FindUSBDevice_setadrok:
	;; Set Target Device Address
	lds	gr0,DEVADDR_CNT
	std	Y+DevAddr,gr0		;; Set Device Addr
	inc	gr0
	sts	DEVADDR_CNT,gr0

	rcall	CheckUniqueDevice	;; P/ECE & USB Writer Check
	cpi	gr0,0xFF
	breq	FindUSBDevice_notuniquedevice
	
	ori	gr0,UNIQUEDEV_STARTINDEX ;; Device type...
	std	Y+DevType,gr0		 ;; SET!

	ldd	gr1,Y+DevNum
	cpi	gr0,PIECE_DEVICE
	brne	FindUSBDevice_notpiece
	sts	PIECEDEVNUM,gr1		;; Set!
	rjmp	FindUSBDevice_notuniquedevice

FindUSBDevice_notpiece:
	cpi	gr0,USBWR_DEVICE
	brne	FindUSBDevice_notuniquedevice
	sts	USBWRDEVNUM,gr1		;; Set!

FindUSBDevice_notuniquedevice:
	rcall	DispDetectMessage	;; display manufacture & product string

;	mov	gr1,gr0
;	rcall	dispresult

	clr	gr0
	ldi	gr2,0x7f
	ldi	gr1,DEV_CONFIGURATION
	rcall	GetDescProbe
;	ldi	gr2,0x12
;	ldi	gr1,DEV_CONFIGURATION
;	rcall	GetDescProbe
;	ldi	gr2,0x09
;	ldi	gr1,DEV_CONFIGURATION
;	rcall	GetDescProbe
;	ldi	gr2,0x22
;	ldi	gr1,DEV_CONFIGURATION
;	rcall	GetDescProbe

	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	ldi	gr0,0x40
	rcall	dump

;; if check Endpoints & configrations ....
	ldd	gr0,Z+cd_bAttr
	std	Y+DAttr,gr0	;; Set Attribute

	rcall	SetDeviceInfo
	cp	gr0,zero
	breq	FindNewUSBDevice_founddev
	rcall	ClearDeviceInfo		;; clear device
	pop	gr1
	ser	gr0
	ret				;; NG

FindNewUSBDevice_founddev:
	ldi	ZH,high(DescriptorBuf)
	ldi	ZL,low(DescriptorBuf)
	ldi	gr0,0x40
	rcall	dump

	;; Configuration Enabled!
	ldi	gr0,DEV_DEVICE
	clr	gr1
	clr	gr2
	clr	ZL
	clr	ZH
	rcall	SetConfig

	pop	gr1
	clr	gr0

	ret

; -------------------------------------------
; Get HID Device Index
; gr0 : Start Index
; gr1 : Search Device(protocol)
; -------------------------------------------
GetHIDDeviceIndex:
	mov	gr2,gr0
	lsl	gr1
	lsl	gr1
	lsl	gr1
	lsl	gr1
	ori	gr1,1		; 
GetKeyDevIndex_loop:
	mov	gr0,gr2
	rcall	GetBaseTblAdr	; Y = Table Address
	ldd	gr0,Y+DevType
	cpi	gr0,HID_DEVICE	; Check HID
	brne	GetKeyDeviceIndex_NoHID
	ldd	gr0,Y+DevSubType
	cp	gr0,gr1
	brne	GetKeyDeviceIndex_NoHID
	ldd	gr0,Y+IntrEP			; Intr Endpoint exist?
	or	gr0,zero
	brne	GetKeyDeviceIndex_Found
GetKeyDeviceIndex_NoHID:
	inc	gr2
	cpi	gr2,MAX_DEVICES
	brcs	GetKeyDevIndex_loop
	ser	gr0
	ret

GetKeyDeviceIndex_Found:
	mov	gr0,gr2
	
	ret

; ------------------------------------------
; Set LED Status
; gr0 : status
; Y : Device Table Address
; ------------------------------------------
SetLEDStatus:
	ldi	ZL,low(DescriptorBuf)
	ldi	ZH,high(DescriptorBuf)
	st	Z,gr0		; set stat
	ldi	gr2,1		; length
	ldi	gr1,OUTPUT_REPORT
	clr	gr0
	rcall	HidSetReport

	ret

; -----------------------------------------
; Conversion Key Code
; Z : Report Data Pointer (for Boot-protocol)
; gr0 : output data
; -----------------------------------------
ConvKeyCode:
	ld	gr0,Z+	;; shift data
	ld	gr1,Z+	;; dummy read
	ld	gr1,Z+	;; key data
	ldi	gr3,0x02
	andi	gr0,0x22
	brne	PC+2
	clr	gr3
	cpi	gr1,0x1e
	brcc	ConvKeyCode_nocheckcaps
	lds	gr2,KEYSTATUS
	andi	gr2,0x02	;; caps
	eor	gr3,gr2	;; if CAPS shiftstatus=!shiftstatus
ConvKeyCode_nocheckcaps:
	ldi	ZL,low(KeyConvTable*2)
	ldi	ZH,high(KeyConvTable*2)
	andi	gr3,0x02
	breq	ConvKeyCode_noshift
	ldi	ZL,low(SftKeyConvTable*2)
	ldi	ZH,high(SftKeyConvTable*2)
ConvKeyCode_noshift:
	add	ZL,gr1
	adc	ZH,zero
	lpm
	mov	gr0,r0

	ret

; -------------------------------------------
; Read Data
; Y : Device Table Address
; Z : Read Buffer Address
; gr5 : ReadSize
; -------------------------------------------
ReadData:
	ldd	gr0,Y+DevAddr
	ldd	gr1,Y+IntrEP
	ldi	gr2,PID_IN
	push	YL
	push	YH
	add	YL,gr1
	adc	YH,zero
	ldd	gr4,Y+DPayload
	pop	YH
	pop	YL
	clr	gr3
;;	ldi	gr5,8	;; for Boot Protocol
	rcall	USBXfer
	
	ret

; -------------------------------------------
; DUMP MEMORY
; Z : start
; gr0 : len
; -------------------------------------------
Dump:
	ret		;; @@debug
	push	gr0
	push	gr1
	push	gr2
	push	gr3
	push	ZH
	push	ZL
	
	mov	gr2,gr0
	rcall	sendcrlf
	
	clr	gr1
dump_loop:
	ld	gr0,Z+
	rcall	BinToHex
	lds	gr0,hexh
	rcall	senddata
	lds	gr0,hexl
	rcall	senddata

	ldi	gr0,' '
	rcall	senddata
	
	inc	gr1
	cp	gr1,gr2
	breq	dump_fin
	mov	gr0,gr1
	andi	gr0,0xf
	brne	dump_loop

	rcall	sendcrlf
	rjmp	dump_loop

dump_fin:
	rcall	sendcrlf

	pop	ZL
	pop	ZH
	pop	gr3
	pop	gr2
	pop	gr1
	pop	gr0

	ret
;
; ------------------------------
; DEBUG OUT
; gr1 : DATA
; ------------------------------
dispresult:
	push	gr0
	push	gr1

	mov	gr0,gr1
	rcall	BinToHex
	lds	gr0,hexh
	rcall	senddata
	lds	gr0,hexl
	rcall	senddata
;;
	rcall	sendcrlf

	pop	gr1
	pop	gr0

	ret

sendcrlf:
	push	gr0
	push	gr3
	ldi	gr0,_CR		;; 
	rcall	senddata
	ldi	gr0,_LF
	rcall	senddata
	pop	gr3
	pop	gr0

	ret

; --------------------------------
; RS232C 1Character Transmit
; gr0 : sendchar
; --------------------------------
senddata:
	push	gr3
senddata_loop:
	in	gr3,USR
	andi	gr3,(1 << UDRE)
	breq	senddata_loop

	out	UDR,gr0
	pop	gr3
	ret

; ---------------------------------
; RS232C 1Character Read
; Carry 0 : gr0  = received data
; Carrt 1 : no data
; --------------------------------
recvdata:
	cli
	lds	gr1,rxrptr
	lds	gr0,rxwptr
	sei

	sub	gr0,gr1
	andi	gr0,0xf
	breq	recvdata_nodata
	mov	ZL,gr1		; read pointer
	clr	ZH
	ld	gr0,Z+		; 
	andi	ZL,0x6f
	sts	rxrptr,ZL
	clc

	ret

recvdata_nodata:
	sec
	ret

; ---------------------------------
; Send ROM Message
; ---------------------------------
sendline:
	lpm
	or	r0,r0
	breq	sendline_fin
	mov	gr0,r0
	rcall	senddata
	add	ZL,one
	adc	ZH,zero
	rjmp	sendline
sendline_fin:
	ret


; ------------------------------------------
; generic wait routine
; 1clk = 4.912MHz (203.58ns)
; ------------------------------------------
wait50us:
	ldi	gr0,81			; 3*81*203.58ns+20 = 50usec wait
wait50u_loop:
	dec	gr0
	brne	wait50u_loop

	ret

waitpre:
	ldi	gr0,41			; 3*81*203.58ns+20 = 50usec wait
waitpre_loop:
	dec	gr0
	brne	waitpre_loop

	ret

wait1ms:
	ldi	gr1,20
wait1ms_loop:
	rcall	wait50us
	dec	gr1
	brne	wait1ms_loop

	ret

wait10ms:
	ldi	gr1,200
wait10ms_loop:
	rcall	wait50us
	dec	gr1
	brne	wait10ms_loop

	ret

wait10xms:
	rcall	wait10ms
	dec	gr2
	brne	wait10xms
	
	ret

;; ---------------------------------
;; Write Data SL811HS
;; ---------------------------------
USBWrite:
	ldi	XH,ADRREQ
	st	X,chipadr
	ldi	XH,DATAREQ
	st	X,chipdata
	
	ret

;; ---------------------------------
;; Write Data SL811HS
;; broken chipadr/chipdata
;; ---------------------------------
USBBufWrite:
	or	chipdata,zero
	brne	PC+2
	ret
	push	ZH
	push	ZL
	
	ldi	XH,ADRREQ
	st	X,chipadr
	ldi	XH,DATAREQ
usbbufwrite_loop:
	ld	chipadr,Z+
	st	X,chipadr
	dec	chipdata
	brne	usbbufwrite_loop
	pop	ZL
	pop	ZH
	
	ret

;; ---------------------------------
;; Read Data SL811HS
;; broken chipadr/chipdata
;; ---------------------------------
USBBufRead:
	or	chipdata,zero
	brne	PC+2
	ret
	push	ZH
	push	ZL
	
	ldi	XH,ADRREQ
	st	X,chipadr
	ldi	XH,DATAREQ
usbbufread_loop:
	ld	chipadr,X
	st	Z+,chipadr
	dec	chipdata
	brne	usbbufread_loop
	pop	ZL
	pop	ZH
	
	ret

; ----------------------------------
; Write Data with Auto Incriment
; XL DATA
; ----------------------------------
USBWriteA:
	ldi	XH,DATAREQ
	st	X,chipdata
	
	ret

; ----------------------------------
; Data Read
; ----------------------------------
USBRead:
	ldi	XH,ADRREQ
	st	X,chipadr
	ldi	XH,DATAREQ
	ld	XL,X
	
	ret

;-----------------------------------
; Auto Data Incriment
;-----------------------------------
USBReadA:
	ldi	XH,DATAREQ
	ld	XL,X
	
	ret

; ------------------------------------
; Binary to HEX
; in  gr0
; out SRAM(hexl & hexh)
; ------------------------------------
BinToHex:
	push	gr0
	andi	gr0,0x0f
	subi	gr0,-0x30
	cpi	gr0,0x3a
	brcs	PC+2
	; if temp>0x39
	subi	gr0,-$7
	sts	hexl,gr0
	pop	gr0
	lsr	gr0
	lsr	gr0
	lsr	gr0
	lsr	gr0
;; high digit
	subi	gr0,-$30
	cpi	gr0,0x3a
	brcs	PC+2
	; if gr0>0x39 
	subi	gr0,-$7
	sts	hexh,gr0

	ret


nodevice_msg:
	.DB	'D','E','V','I','C','E',' ','N','O','T',' ','F','O','U','N','D',_CR,_LF,0

disconnect_msg:
	.DB	'D','E','V','I','C','E',' ','D','I','S','C','O','N','N','E','C','T','E','D',_CR,_LF,0

product_msg:
	.DB	'P','r','o','d','u','c','t',' ','N','a','m','e',':',' ',0

manufacture_msg:
	.DB	'N','e','w',' ','D','e','v','i','c','e',' ','F','o','u','n','d','.',' ',_CR,_LF
	.DB	'M','a','n','u','f','a','c','t','u','r','e',' ','N','a','m','e',':',' ',0

noname_msg:
	.DB	'<','N','O',' ','N','A','M','E','>',_CR,_LF,0

;; Device ID Tables
devid_table:
	.DB	PIECE_MID_L,PIECE_MID_H,PIECE_PID_L,PIECE_PID_H
	.DB	USBWR_MID_L,USBWR_MID_H,USBWR_PID_L,USBWR_PID_H
	.DB	0xFF,0xFF,0xFF,0xFF

;; NO-SHIFT (for 106 Keyboard)
;; CAPS LOCK = 0xFE
;; NUM LOCK = 0xFD
;; SCROLL LOCK = 0xFC
;; BS        = 0x08
KeyConvTable:
;		 +0   +1   +2   +3   +4   +5   +6   +7 
;		 +8   +9   +A   +B   +C   +D   +E   +F
	.DB	0xFF,0xFF,0xFF,0xFF, 'a', 'b', 'c', 'd'
	.DB	 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l'
; 0x10 - 0x1F
	.DB	 'm', 'n', 'o', 'p', 'q', 'r', 's', 't'
	.DB	 'u', 'v', 'w', 'x', 'y', 'z', '1', '2'
; 0x20 - 0x2F
	.DB	 '3', '4', '5', '6', '7', '8', '9', '0'
	.DB	0x0D,0x1B,0x08,0x09, ' ', '-', '^', '@'
; 0x30 - 0x3F
	.DB	 '[', '\', ']', 0x3b, ':',0xFF, ',', '.'
	.DB	 '/',0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0x40 - 0x4F
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFC
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x1C
; 0x50 - 0x5F
	.DB	0x1D,0x1F,0x1E,0xFD, '/', '*', '-', '+'
	.DB	0x0D, '1', '2', '3', '4', '5', '6', '7'
; 0x60 - 0x6F
	.DB	 '8', '9', '0', '.',0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0x70 - 0x7F
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0x80 - 0x8F
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, '\'
	.DB	0xFF, '\',0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0x90 - 0x9F
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xA0 - 0xAF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xB0 - 0xBF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xC0 - 0xCF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xD0 - 0xDF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xE0 - 0xEF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xFF - 0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF

;; SHIFT (for 106 Keyboard)
SftKeyConvTable:
;		 +0   +1   +2   +3   +4   +5   +6   +7 
;		 +8   +9   +A   +B   +C   +D   +E   +F
	.DB	0xFF,0xFF,0xFF,0xFF, 'A', 'B', 'C', 'D'
	.DB	 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L'
; 0x10 - 0x1F
	.DB	 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T'
	.DB	 'U', 'V', 'W', 'X', 'Y', 'Z', '!', '"'
; 0x20 - 0x2F
	.DB	 '#', '$', '%', '&',0x27, '(', ')', 0xFF
	.DB	0x0D,0x1B,0x08,0x09, ' ', '=', '~', '`'
; 0x30 - 0x3F
	.DB	 '{', '\', '}', '+', '*', 0xFF, '<', '>'
	.DB	 '?',0xFE,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0x40 - 0x4F
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFC
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x1C
; 0x50 - 0x5F
	.DB	0x1D,0x1F,0x1E,0xFD, '/', '*', '-', '+'
	.DB	0x0D, '1', '2', '3', '4', '5', '6', '7'
; 0x60 - 0x6F
	.DB	 '8', '9', '0', '.',0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0x70 - 0x7F
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0x80 - 0x8F
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, '_'
	.DB	0xFF, '|',0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0x90 - 0x9F
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xA0 - 0xAF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xB0 - 0xBF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xC0 - 0xCF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xD0 - 0xDF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xE0 - 0xEF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
; 0xFF - 0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF

	.DSEG
; internal SRAM area

	.org	0x60

rxbuf:	.BYTE	16		; 16byte buffer	(MUST BE HERE!!)
rxwptr:  .BYTE	1		; write pointer
rxrptr:  .BYTE	1		; read pointer

hexl:	.BYTE	1
hexh:	.BYTE	1

DEVADDR_CNT: .BYTE 1

; Free-run timer counter
timercnt:
	 .BYTE	1

;; work
rLen:	.BYTE	1
bLen:	.BYTE	1
rem:	.BYTE	1
addr:	.BYTE	1
result:	.BYTE	1
timeout: .BYTE  1
DDD:	.BYTE	1
p1:	.BYTE	1
enablenak:	.BYTE	1
full_speed:	.BYTE	1	
disconnected:	.BYTE	1

;; SETUP Request Buffer
SETUPBUF:
bmRequest:	.BYTE	1
bRequest:	.BYTE	1
wValue:		.BYTE	2
wIndex:		.BYTE	2
wLength:	.BYTE	2

HubWait:	.BYTE	1
HubDevNum:	.BYTE	1
HubPortNum:	.BYTE	1
HubPorts:	.BYTE	1

PIECEDEVNUM:	.BYTE	1
USBWRDEVNUM:	.BYTE	1

KEYSTATUS:	.BYTE	1
STATUSCNT:	.BYTE	1

; Previous Key Code
PrevKey:	.BYTE	MAX_DEVICES
; Previous Mouse Button Status
PrevButton:	.BYTE	(MAX_DEVICES*3)

; Interrupt Interval Counter(CONST)
RstCounters:	.BYTE	MAX_DEVICES
; Interrupt Interval Counter(Val)
IntrCounters:	.BYTE	MAX_DEVICES

;; Device Informations
DEVICE_TABLE:
devNum0:	.BYTE	1	; Device Number
devType0:	.BYTE	1	; Device Type
devTypeSub0:	.BYTE	1	; Device Sub Type
IntrEP0:	.BYTE	1	; Interrupt Ep
DevAddr0:	.BYTE	1	; Device Address
DevSpeed0:	.BYTE	1	; Device Speed
HubPorts0:	.BYTE	1	; Connected Hub Port
DevEP0:		.BYTE	ENDPOINTS	; Endpoint
Attr0:		.BYTE	ENDPOINTS	; Attr
Interval0:	.BYTE	ENDPOINTS	; Interval
Payload0:	.BYTE	ENDPOINTS	; EP Status
DToggle0:	.BYTE	ENDPOINTS	; Toggle
DEVICE_TABLE_BTM:

DEVICE_TABLE_D: .BYTE	DEV_TABLE_SIZE_ALL


DescriptorBuf:	.BYTE	128	; Descriper/Interface Receive Buffer

	.org	0x240
__stack:	.BYTE	32	; stack 32byte
